@@ -71,6 +71,7 @@ GlobPreferences::GlobPreferences(void)
     default_download_signed(true),
     //store_passwords_on_disk(false),
     store_messages_on_disk(true),
+    toolbar_button_style(Qt::ToolButtonTextUnderIcon),
     store_additional_data_on_disk(true),
     certificate_validation_date(DOWNLOAD_DATE),
     check_crl(true),
@@ -145,6 +146,10 @@ void GlobPreferences::loadFromSettings(const QSettings &settings)
 	    "preferences/download_at_start",
 	    dlftlGlobPref.download_at_start).toBool();
 
+	toolbar_button_style = settings.value(
+	    "preferences/toolbar_button_style",
+	    dlftlGlobPref.toolbar_button_style).toInt();
+
 	timer_value = settings.value(
 	    "preferences/timer_value", dlftlGlobPref.timer_value).toInt();
 
@@ -156,6 +161,10 @@ void GlobPreferences::loadFromSettings(const QSettings &settings)
 	    "preferences/timestamp_expir_before_days",
 	    dlftlGlobPref.timestamp_expir_before_days).toInt();
 
+	message_mark_as_read_timeout = settings.value(
+	    "preferences/message_mark_as_read_timeout",
+	    dlftlGlobPref.message_mark_as_read_timeout).toInt();
+
 	use_global_paths = settings.value(
 	    "preferences/use_global_paths",
 	    dlftlGlobPref.use_global_paths).toBool();
@@ -347,6 +356,10 @@ void GlobPreferences::saveToSettings(QSettings &settings) const
 		settings.setValue("after_start_select", after_start_select);
 	}
 
+	if (dlftlGlobPref.toolbar_button_style != toolbar_button_style) {
+		settings.setValue("toolbar_button_style", toolbar_button_style);
+	}
+
 	if (dlftlGlobPref.timer_value != timer_value) {
 		settings.setValue("timer_value", timer_value);
 	}
@@ -363,6 +376,12 @@ void GlobPreferences::saveToSettings(QSettings &settings) const
 		    timestamp_expir_before_days);
 	}
 
+	if (dlftlGlobPref.message_mark_as_read_timeout !=
+	    message_mark_as_read_timeout) {
+		settings.setValue("message_mark_as_read_timeout",
+		    message_mark_as_read_timeout);
+	}
+
 	if (dlftlGlobPref.download_on_background != download_on_background) {
 		settings.setValue("download_on_background",
 		    download_on_background);
@@ -1170,6 +1189,22 @@ QString toBase64(const QString &plain)
 	return QString::fromUtf8(plain.toUtf8().toBase64());
 }
 
+/* ========================================================================= */
+int base64RealSize(const QByteArray &b64)
+/* ========================================================================= */
+{
+	int b64size = b64.size();
+	int cnt = 0;
+	if (b64size >= 3) {
+		for (int i = 1; i <= 3; ++i) {
+			if ('=' == b64[b64size - i]) {
+				++cnt;
+			}
+		}
+	}
+	return b64size * 3 / 4 - cnt;
+}
+
 
 #define CREDITS_FILE "AUTHORS"
 #define LICENCE_FILE "COPYING"
@@ -1409,7 +1444,11 @@ QString writeTemporaryFile(const QString &fileName, const QByteArray &data,
 		return QString();
 	}
 
-	QTemporaryFile fout(QDir::tempPath() + QDir::separator() + fileName);
+	QString nameCopy(fileName);
+	nameCopy.replace(QRegExp("[" + QRegExp::escape( "\\/:*?\"<>|" ) + "]"),
+	    QString( "_" ));
+
+	QTemporaryFile fout(QDir::tempPath() + QDir::separator() + nameCopy);
 	if (!fout.open()) {
 		return QString();
 	}
@@ -1437,29 +1476,26 @@ QString writeTemporaryFile(const QString &fileName, const QByteArray &data,
 /*
  * Create filename based on format string.
  */
-QString createFilenameFromFormatString(QString pattern,
-    QDateTime dmAcceptanceTime, QString dmAnnotation, QString dmID,
-    QString dbID, QString usename, QString attachFilename)
+QString createFilenameFromFormatString(QString pattern, const QString &dmID,
+    const QString &dbID, const QString &userName, const QString &attachFilename,
+    const QDateTime &dmDeliveryTime, QDateTime dmAcceptanceTime,
+    QString dmAnnotation, QString dmSender)
 /* ========================================================================= */
 {
 	debugFuncCall();
 
-	// known atrributes
-	// {"%Y","%M","%D","%h","%m","%i","%s","%d","%u","%f"};
+	// for future use
+	(void) dmDeliveryTime;
 
-	if (pattern.isEmpty() || pattern.isNull()) {
+	// known atrributes
+	// {"%Y","%M","%D","%h","%m","%i","%s","%S","%d","%u","%f"};
+
+	if (pattern.isEmpty()) {
 		pattern = DEFAULT_TMP_FORMAT;
 	}
-/*
-	if (attachFilename.isEmpty() || attachFilename.isNull()) {
-		attachFilename = tr("filename");
-	} else {
-		QFileInfo filename(attachFilename);
-		attachFilename = filename.completeBaseName();
-	}
-*/
+
 	if (!dmAcceptanceTime.isValid()) {
-		dmAcceptanceTime.currentDateTime();
+		dmAcceptanceTime = QDateTime::currentDateTime();
 	}
 
 	QPair<QString, QString> pair;
@@ -1486,11 +1522,14 @@ QString createFilenameFromFormatString(QString pattern,
 	pair.first = "%s";
 	pair.second = dmAnnotation.replace(" ","-");
 	knowAtrrList.append(pair);
+	pair.first = "%S";
+	pair.second = dmSender.replace(" ","-");
+	knowAtrrList.append(pair);
 	pair.first = "%d";
 	pair.second = dbID;
 	knowAtrrList.append(pair);
 	pair.first = "%u";
-	pair.second = usename;
+	pair.second = userName;
 	knowAtrrList.append(pair);
 	pair.first = "%f";
 	pair.second = attachFilename;
@@ -1500,5 +1539,44 @@ QString createFilenameFromFormatString(QString pattern,
 		pattern.replace(knowAtrrList[i].first, knowAtrrList[i].second);
 	}
 
+	/*
+	* Eliminate illegal characters "\/:*?"<>|" in the filename.
+	* All these characters are replaced by "_".
+	*/
+	pattern.replace(QRegExp("[" + QRegExp::escape( "\\/:*?\"<>|" ) + "]"),
+	    QString( "_" ));
+
 	return pattern;
 }
+
+
+/* ========================================================================= */
+/*
+ * Check if file exists in the path and return non-conflicting filename.
+ */
+QString getNonConflictingFileName(QString fileName)
+/* ========================================================================= */
+{
+	debugFuncCall();
+
+	if (QFile::exists(fileName)) {
+
+		int fileCnt = 0;
+		QFileInfo fi(fileName);
+		// remember origin filename and path
+		const QString base(fi.baseName());
+		const QString path(fi.path());
+		const QString suffix(fi.completeSuffix());
+
+		do {
+			fileCnt++;
+			// create a new filename
+			QString newName(base + "_" + QString::number(fileCnt));
+			// join path and new filename
+			fileName = path + QString(QDir::separator())
+			    + newName + "." + suffix;
+		} while (QFile::exists(fileName));
+	}
+
+	return fileName;
+}
