@@ -39,6 +39,7 @@
 #include "src/common.h"
 #include "src/io/account_db.h"
 #include "src/io/message_db.h"
+#include "src/io/message_db_set.h"
 #include "src/gui/dlg_import_zfo.h"
 #include "src/gui/dlg_timestamp_expir.h"
 #include "src/models/accounts_model.h"
@@ -71,7 +72,7 @@ public:
 		QString databoxID;
 		QString accountName;
 		QString username;
-		MessageDb *messageDb;
+		MessageDbSet *messageDbSet;
 		QModelIndex acntIndex;
 	};
 
@@ -82,6 +83,57 @@ public:
 		MSG_FILE_ERROR
 	};
 
+	/*!
+	 * @brief Create configuration file if not present.
+	 */
+	static
+	bool ensureConfPresence(void);
+
+	/*!
+	 * @brief connect to ISDS databox from exist account
+	 *
+	 * @note If pointer to main windows is specified, then various
+	 *     dialogues may be shown. Passwords are used only when required
+	 *     (i.e. not found in configuration).
+	 *
+	 * @param userName Account login.
+	 * @param mw       Pointer to main window.
+	 * @param pwd      User password.
+	 * @param otpKey   One-time password or certificate key.
+	 */
+	static
+	bool connectToIsds(const QString &userName, MainWindow *mw,
+	    const QString &pwd = QString(), const QString &otpKey = QString());
+
+	/*!
+	 * @brief Get message db set related to given account.
+	 *
+	 * @note If pointer to main window is not specified, then all dialogues
+	 *     will be suppressed.
+	 *
+	 * @param userName Account login.
+	 * @param mw       Pointer to main window.
+	 */
+	static
+	MessageDbSet *accountDbSet(const QString &userName, MainWindow *mw);
+
+	/*!
+	 * @brief Get data about logged in user and his box.
+	 */
+	static
+	bool getOwnerInfoFromLogin(const QString &userName);
+
+	/*!
+	 * @brief Get information about password expiration date.
+	 */
+	static
+	bool getPasswordInfoFromLogin(const QString &userName);
+
+	/*!
+	 * @brief Get data about logged in user.
+	 */
+	static
+	bool getUserInfoFromLogin(const QString &userName);
 
 protected:
 	/*!
@@ -105,11 +157,15 @@ private slots:
 	void datovkaVersionResponce(QNetworkReply* reply);
 
 	/*!
-	 * @brief Verify if is a connection to ISDS and databox
-	 * exists for a new account.
+	 * @brief Updates the account model according to the change properties.
 	 */
-	void getAccountUserDataboxInfo(
-	    const AccountModel::SettingsMap &accountInfo);
+	void updateAccountListEntry(const QString &userName);
+
+	/*!
+	 * @brief Verify whether a connection to ISDS can be establisehd
+	 *     and databox exists for the account.
+	 */
+	void getAccountUserDataboxInfo(AccountModel::SettingsMap accountInfo);
 
 	/*!
 	 * @brief Redraws widgets according to selected account item.
@@ -165,7 +221,7 @@ private slots:
 	 *        message ID from search selection
 	 */
 	void messageItemFromSearchSelection(const QString &userName,
-	    qint64 msgId);
+	    qint64 msgId, const QString &deliveryYear, int msgType);
 
 	/*!
 	 * @brief Redraws widgets according to selected attachment item.
@@ -357,7 +413,7 @@ private slots:
 	/*!
 	 * @brief Verifies selected message and creates response dialog.
 	 */
-	void verifyMessage(void);
+	void verifySelectedMessage(void);
 
 	/*!
 	 * @brief Shows account properties dialog.
@@ -394,6 +450,11 @@ private slots:
 	 */
 	void prepareCreateAccountFromDatabaseFile(bool fromDirectory);
 
+	/*!
+	 * @brief Prepare import of messages from database.
+	 */
+	void prepareMsgsImportFromDatabase(void);
+
 	/*!
 	 * @brief Proxy setting dialog.
 	 */
@@ -423,29 +484,30 @@ private slots:
 	 * @brief Export message into as ZFO file dialog.
 	 */
 	void exportSelectedMessageAsZFO(const QString &attachPath = QString(),
-	    qint64 dmID = -1, const QStandardItem *accountItem = NULL);
+	    QString userName = QString(), qint64 dmID = -1,
+	    const QDateTime &delivTime = QDateTime());
 
 	/*!
 	 * @brief Export delivery information as ZFO file dialog.
 	 */
 	void exportDeliveryInfoAsZFO(const QString &attachPath = QString(),
-	    QString attachFileName = QString(),
-	    QString formatString = globPref.delivery_filename_format,
-	    qint64 dmID = -1);
+	    const QString &attachFileName = QString(),
+	    const QString &formatString = globPref.delivery_filename_format,
+	    qint64 dmID = -1, const QDateTime &delivTime = QDateTime());
 
 	/*!
 	 * @brief Export delivery information as PDF file dialog.
 	 */
 	void exportDeliveryInfoAsPDF(const QString &attachPath = QString(),
-	    QString attachFileName = QString(),
-	    QString formatString = globPref.delivery_filename_format,
-	    qint64 dmID = -1);
+	    const QString &attachFileName = QString(),
+	    const QString &formatString = globPref.delivery_filename_format,
+	    qint64 dmID = -1, const QDateTime &delivTime = QDateTime());
 
 	/*!
 	 * @brief Export selected message envelope as PDF file dialog.
 	 */
 	void exportMessageEnvelopeAsPDF(const QString &attachPath = QString(),
-	    qint64 dmID = -1);
+	    qint64 dmID = -1, const QDateTime &delivTime = QDateTime());
 
 	/*!
 	 * @brief Open selected message in external application.
@@ -545,10 +607,15 @@ private slots:
 	void aboutApplication(void);
 
 	/*!
-	 * @brief Show help.
+	 * @brief Show help/manual in a internet browser.
 	 */
 	void showHelp(void);
 
+	/*!
+	 * @brief Go to homepage.
+	 */
+	void goHome(void);
+
 	/*!
 	 * @brief Clear message filter field.
 	 */
@@ -577,7 +644,7 @@ private slots:
 	/*!
 	 * @brief Refresh AccountList.
 	 */
-	void refreshAccountListFromWorker(const QModelIndex acntTopIdx);
+	void refreshAccountListFromWorker(const QString &userName);
 
 	/*!
 	 * @brief Set and run any actions after main window has been created.
@@ -594,8 +661,8 @@ private slots:
 	/*!
 	 * @brief Set tablewidget when message download worker is done.
 	 */
-	void postDownloadSelectedMessageAttachments(
-	    const QModelIndex &acntTopIdx, qint64 dmId);
+	void postDownloadSelectedMessageAttachments(const QString &userName,
+	    qint64 dmId);
 
 	/*!
 	 * @brief Set info status bar from worker.
@@ -628,6 +695,11 @@ private slots:
 	 */
 	void prepareMsgTmstmpExpir(enum TimestampExpirDialog::TSaction action);
 
+	/*!
+	 * @brief Split message database slot.
+	 */
+	void splitMsgDbByYearsSlot(void);
+
 private:
 
 	QThread *m_syncAcntThread;
@@ -635,11 +707,26 @@ private:
 	QTimer m_timerSyncAccounts;
 	int m_timeoutSyncAccounts;
 
+	void showStatusTextWithTimeout(const QString &qStr);
+
+	void showStatusTextPermanently(const QString &qStr);
+
+	/*!
+	 * @brief Return index for yearly entry with given properties.
+	 */
+	QModelIndex accountYearlyIndex(const QString &userName,
+	    const QString &year, int msgType);
+
+	/*!
+	 * @brief Return index for message with given properties.
+	 */
+	QModelIndex messageIndex(qint64 msgId) const;
+
 	/*!
 	 * @brief Check message time stamp expiration for account.
 	 */
-	void checkMsgsTmstmpExpiration(const QStandardItem *accountItem,
-	    const QString &accountName, QStringList filePathList);
+	void checkMsgsTmstmpExpiration(const QString &userName,
+	    const QStringList &filePathList);
 
 	/*!
 	 * @brief Mark all received messages in the current working account.
@@ -712,11 +799,6 @@ private:
 	 */
 	void activeAccountMenuAndButtons(bool action) const;
 
-	/*!
-	 * @brief Create configuration file if not present.
-	 */
-	bool ensureConfPresence(void) const;
-
 	/*!
 	 * @brief Load and apply setting from configuration file.
 	 */
@@ -827,14 +909,22 @@ private:
 	/*!
 	 * @brief Generate account info HTML message.
 	 */
-	QString createAccountInfo(const QStandardItem &topItem);
+	QString createAccountInfo(const QString &userName);
 
 	/*!
 	 * @brief Generate overall account information.
 	 */
 	QString createAccountInfoAllField(const QString &accountName,
 	    const QList< QPair<QString, int> > &receivedCounts,
-	    const QList< QPair<QString, int> > &sent) const;
+	    const QList< QPair<QString, int> > &sentCounts) const;
+
+	/*!
+	 * @brief Generate overall account information only for sent or
+	 *     received messages.
+	 */
+	QString createAccountInfoMessagesCount(const QString &accountName,
+	    const QList< QPair<QString, int> > &counts,
+	    enum MessageDb::MessageType type) const;
 
 	/*!
 	 * @brief Generate banner.
@@ -843,39 +933,37 @@ private:
 
 	/*!
 	 * @brief Returns user name related to given account item.
-	 * TODO - If accountItem is 0 then currently selected account is
-	 *     processed.
-	 */
-	QString accountUserName(const QStandardItem *accountItem = 0) const;
-
-	/*!
-	 * @brief Get message db to selected account item.
 	 *
 	 * @note If accountItem is 0 then currently selected account is
 	 *     processed.
 	 */
-	MessageDb * accountMessageDb(const QStandardItem *accountItem);
+	QString userNameFromItem(const QStandardItem *accountItem = 0) const;
+
+	/*!
+	 * @brief Returns pointer to account item related to given user name.
+	 *
+	 * @param[in] userName User login name.
+	 * @return 0 when no such top item found.
+	 */
+	QStandardItem *itemFromUserName(const QString &userName) const;
 
 	/*!
 	 * @brief Set export paths to selected account item.
-	 *
-	 * @note If accountItem is 0 then currently selected account is
-	 *     processed.
 	 */
-	void setAccountStoragePaths(const QStandardItem *accountItem);
+	void setAccountStoragePaths(const QString &userName);
 
 	/*!
 	 * @brief Delete message from long term storage in ISDS and
 	 * local database - based on delFromIsds parameter.
 	 */
-	qdatovka_error eraseMessage(const QModelIndex &acntTopIdx,
-	    qint64 dmId, bool delFromIsds);
+	qdatovka_error eraseMessage(const QString &userName, qint64 dmId,
+	    const QDateTime &deliveryTime, bool delFromIsds);
 
 	/*!
 	 * @brief Verify message. Compare hash with hash stored in ISDS.
 	 */
-	qdatovka_error verifySelectedMessage(const QModelIndex &acntTopIdx,
-	    const QModelIndex &msgIdx);
+	qdatovka_error verifyMessage(const QString &userName, qint64 dmId,
+	    const QDateTime &deliveryTime);
 
 	/*!
 	 * @brief Authenticate message from ZFO file.
@@ -891,31 +979,32 @@ private:
 	/*!
 	 * @brief Check if connection to ISDS fails.
 	 */
+	static
 	bool checkConnectionError(int status, const QString &accountName,
-	    bool showDialog, const QString &isdsMsg);
-
-	/*!
-	 * @brief connect to ISDS databox from exist account
-	 */
-	bool connectToIsds(const QModelIndex &acntTopIdx, bool showDialog);
+	    const QString &isdsMsg, MainWindow *mw);
 
 	/*!
 	 * @brief connect to ISDS databox from new account
 	 */
-	bool firstConnectToIsds(const AccountModel::SettingsMap &accountInfo,
+	bool firstConnectToIsds(AccountModel::SettingsMap &accountInfo,
 	    bool showDialog);
 
 	/*!
-	 * @brief Login to ISDS server by username and password only.
+	 * @brief Login to ISDS server by user name and password only.
 	 */
-	bool loginMethodUserNamePwd(const QModelIndex &acntTopIdx,
-	   const AccountModel::SettingsMap &accountInfo, bool showDialog);
+	static
+	bool loginMethodUserNamePwd(
+	    AccountModel::SettingsMap &accountInfo,
+	    MainWindow *mw, const QString &pwd = QString());
 
 	/*!
-	 * @brief Login to ISDS server by username, password and OTP code.
+	 * @brief Login to ISDS server by user name, password and OTP code.
 	 */
-	bool loginMethodUserNamePwdOtp(const QModelIndex &acntTopIdx,
-	    const AccountModel::SettingsMap &accountInfo, bool showDialog);
+	static
+	bool loginMethodUserNamePwdOtp(
+	    AccountModel::SettingsMap &accountInfo,
+	    MainWindow *mw, const QString &pwd = QString(),
+	    const QString &otp = QString());
 
 	/*!
 	 * @brief Converts PKCS #12 certificate into PEM format.
@@ -938,20 +1027,25 @@ private:
 	/*!
 	 * @brief Login to ISDS server by certificate only.
 	 */
-	bool loginMethodCertificateOnly(const QModelIndex &acntTopIdx,
-	    const AccountModel::SettingsMap &accountInfo, bool showDialog);
+	static
+	bool loginMethodCertificateOnly(AccountModel::SettingsMap &accountInfo,
+	    MainWindow *mw, const QString &key = QString());
 
 	/*!
-	 * @brief Login to ISDS server by certificate, username and password.
+	 * @brief Login to ISDS server by certificate, user name and password.
 	 */
-	bool loginMethodCertificateUserPwd(const QModelIndex &acntTopIdx,
-	    const AccountModel::SettingsMap &accountInfo, bool showDialog);
+	static
+	bool loginMethodCertificateUserPwd(
+	    AccountModel::SettingsMap &accountInfo,
+	    MainWindow *mw, const QString &pwd = QString(),
+	    const QString &key = QString());
 
 	/*!
-	 * @brief Login to ISDS server by certificate and databox ID.
+	 * @brief Login to ISDS server by certificate and data box ID.
 	 */
-	bool loginMethodCertificateIdBox(const QModelIndex &acntTopIdx,
-	    const AccountModel::SettingsMap &accountInfo, bool showDialog);
+	static
+	bool loginMethodCertificateIdBox(
+	    AccountModel::SettingsMap &accountInfo, MainWindow *mw);
 
 	/*!
 	 * @brief Sent and check a new version of Datovka.
@@ -968,7 +1062,8 @@ private:
 	 * @brief Download complete message synchronously
 	 * without worker and thread.
 	 */
-	bool downloadCompleteMessage(qint64 dmId);
+	bool downloadCompleteMessage(qint64 dmId,
+	    const QDateTime &deliveryTime);
 
 	/*!
 	 * @brief Set read status to messages with given indexes.
@@ -989,34 +1084,64 @@ private:
 	int showDialogueAboutPwdExpir(const QString &accountName,
 	    const QString &userName, qint64 days, const QDateTime &dateTime);
 
-	/*
-	 * @brief Get data about logged in user and his box.
-	 */
-	bool getOwnerInfoFromLogin(const QModelIndex &acntTopIdx,
-	    const QString &userName);
-
-	/*!
-	 * @brief Get information about password expiration date.
-	 */
-	bool getPasswordInfoFromLogin(const QModelIndex &acntTopIdx,
-	    const QString &userName);
-
 	/*!
 	 * @brief Get information about remaining PDZ credit.
 	 */
 	QString getPDZCreditFromISDS(void);
 
 	/*!
-	 * @brief Get data about logged in user.
+	 * @brief Export message with expired time stamp to ZFO.
 	 */
-	bool getUserInfoFromLogin(const QModelIndex &acntTopIdx,
+	void exportExpirMessagesToZFO(const QString &userName,
+	    const QList<MessageDb::MsgId> &expirMsgIds);
+
+	/*!
+	 * @brief Import of messages from database to selected account.
+	 */
+	void doMsgsImportFromDatabase(const QStringList &dbFileList,
 	    const QString &userName);
 
 	/*!
-	 * @brief Export message with expired time stamp to ZFO.
+	 * @brief Split database filename into mandatory entries.
+	 *
+	 * @param[in] inDbFileName - input database file name.
+	 * @param[out] dbUserName - username entry.
+	 * @paran[out] dbYear - year entry if exists or NULL.
+	 * @paran[out] dbTestingFlag - true if account is testing or false
+	 * @paran[out] errMsg - error message to user
+	 * @return true if database filename is correct
 	 */
-	void exportExpirMessagesToZFO(const QStringList &expirMsg,
-	    const QStandardItem *accountItem);
+	bool isValidDatabaseFileName(QString inDbFileName, QString &dbUserName,
+	    QString &dbYear, bool &dbTestingFlag, QString &errMsg);
+
+	/*!
+	 * @brief Split message database into new databases
+	 * contain messages for single years.
+	 * @param[in] userName - username of account.
+	 * @return true if success
+	 */
+	bool splitMsgDbByYears(const QString &userName);
+
+	/*!
+	 * @brief Show error message box
+	 * @param[in] msgTitle - title of message box.
+	 * @param[in] msgText - main text of message box.
+	 * @param[in] msgInformativeText - info text of message box.
+	 */
+	void showErrMessageBox(const QString &msgTitle,
+	    const QString &msgText, const QString &msgInformativeText);
+
+	/*!
+	 * @brief Set back original database path if error during
+	 *    database splitting
+	 * @param[in] dbset - MessageDbSet.
+	 * @param[in] dbDir - origin database dir.
+	 * @param[in] userName - username of account.
+	 * @return true if success
+	 */
+	bool setBackOriginDb(MessageDbSet *dbset, const QString &dbDir,
+	    const QString &userName);
+
 
 	QString m_confDirName; /*!< Configuration directory location. */
 	QString m_confFileName; /*!< Configuration file location. */
@@ -1025,8 +1150,6 @@ private:
 	                              * Account tree view model. Generated from
 	                              * configuration file.
 	                              */
-	AccountDb m_accountDb; /*!< Account information database. */
-	DbContainer m_messageDbs; /*!< Map of message databases. */
 	QLineEdit *m_filterLine; /*!< Search filter line object. */
 	QPushButton *m_clearFilterLineButton; /*!< Button object. */
 	SortFilterProxyModel m_messageListProxyModel; /*!<
