@@ -29,12 +29,11 @@
 #include "src/models/accounts_model.h"
 
 
-DlgChangePwd::DlgChangePwd(const QString &boxId, QTreeView &accountList,
-    const AccountModel::SettingsMap &accountInfo, QWidget *parent)
+DlgChangePwd::DlgChangePwd(const QString &boxId, const QString &userName,
+    QWidget *parent)
     : QDialog(parent),
-    m_accountList(accountList),
-    m_accountInfo(accountInfo),
-    m_boxId(boxId)
+    m_boxId(boxId),
+    m_userName(userName)
 {
 	setupUi(this);
 	initPwdChangeDialog();
@@ -48,11 +47,13 @@ DlgChangePwd::DlgChangePwd(const QString &boxId, QTreeView &accountList,
 void DlgChangePwd::initPwdChangeDialog(void)
 /* ========================================================================= */
 {
+	this->userNameLneEdit->setText(m_userName);
 	this->accountLineEdit->setText(m_boxId);
 	connect(this->generateButton, SIGNAL(clicked()), this,
 	    SLOT(generatePassword()));
 	connect(this->showHideButton, SIGNAL(clicked()), this,
 	    SLOT(showHidePasswordLine()));
+
 	this->buttonBox->button(QDialogButtonBox::Ok)->setEnabled(false);
 	connect(this->newPwdLineEdit, SIGNAL(textChanged(QString)),
 	    this, SLOT(checkInputFields()));
@@ -60,16 +61,28 @@ void DlgChangePwd::initPwdChangeDialog(void)
 	    this, SLOT(checkInputFields()));
 	connect(this->NewPwdLineEdit2, SIGNAL(textChanged(QString)),
 	    this, SLOT(checkInputFields()));
+	connect(this->secCodeLineEdit, SIGNAL(textChanged(QString)),
+	    this, SLOT(checkInputFields()));
 	connect(this->buttonBox, SIGNAL(accepted()), this,
 	    SLOT(changePassword(void)));
 
-	if (m_accountInfo.loginMethod() == LIM_HOTP ||
-	    m_accountInfo.loginMethod() == "topt") {
+	this->secCodeLineEdit->setEnabled(false);
+	this->otpLabel->setEnabled(false);
+	this->smsPushButton->setEnabled(false);
+
+	if (AccountModel::globAccounts[m_userName].loginMethod() == LIM_HOTP) {
 		this->secCodeLineEdit->setEnabled(true);
-		this->label_7->setEnabled(true);
-	} else {
-		this->secCodeLineEdit->setEnabled(false);
-		this->label_7->setEnabled(false);
+		this->otpLabel->setText(tr("Enter security code:"));
+		this->otpLabel->setEnabled(true);
+	}
+
+	if (AccountModel::globAccounts[m_userName].loginMethod() == LIM_TOTP) {
+		this->secCodeLineEdit->setEnabled(true);
+		this->smsPushButton->setEnabled(true);
+		this->otpLabel->setText(tr("Enter SMS code:"));
+		this->otpLabel->setEnabled(true);
+		connect(this->smsPushButton, SIGNAL(clicked()), this,
+		    SLOT(sendSmsCode()));
 	}
 
 	pingTimer = new QTimer(this);
@@ -87,7 +100,7 @@ void DlgChangePwd::initPwdChangeDialog(void)
 void DlgChangePwd::pingIsdsServer(void)
 /* ========================================================================= */
 {
-	if (isdsSessions.isConnectedToIsds(m_accountInfo.userName())) {
+	if (isdsSessions.isConnectedToIsds(m_userName)) {
 		qDebug() << "Connection to ISDS is alive :)";
 	} else {
 		qDebug() << "Connection to ISDS is dead :(";
@@ -115,12 +128,14 @@ QString DlgChangePwd::generateRandomString(void)
 /* ========================================================================= */
 {
 	QString randomString;
+
 	for(int i=0; i<randomStringLength; ++i) {
 		int index = qrand() % possibleCharacters.length();
 		QChar nextChar = possibleCharacters.at(index);
 		randomString.append(nextChar);
 	}
-	return randomString;
+	/* set one digit as last char */
+	return randomString + "0";
 }
 
 
@@ -133,10 +148,19 @@ void DlgChangePwd::checkInputFields(void)
 {
 	bool buttonEnabled = !this->currentPwdLineEdit->text().isEmpty() &&
 	    !this->newPwdLineEdit->text().isEmpty() &&
-	    this->newPwdLineEdit->text().length() > 7 &&
+	    this->newPwdLineEdit->text().length() >= PWD_MIN_LENGTH &&
 	    !this->NewPwdLineEdit2->text().isEmpty() &&
-	    this->NewPwdLineEdit2->text().length() > 7 &&
+	    this->NewPwdLineEdit2->text().length() >= PWD_MIN_LENGTH &&
 	    this->NewPwdLineEdit2->text() == this->newPwdLineEdit->text();
+
+	Q_ASSERT(!m_userName.isEmpty());
+
+	if (AccountModel::globAccounts[m_userName].loginMethod() == LIM_HOTP ||
+	    AccountModel::globAccounts[m_userName].loginMethod() == LIM_TOTP) {
+		buttonEnabled = buttonEnabled &&
+		    !this->secCodeLineEdit->text().isEmpty();
+	}
+
 	this->buttonBox->button(QDialogButtonBox::Ok)->setEnabled(
 	    buttonEnabled);
 }
@@ -161,6 +185,81 @@ void DlgChangePwd::showHidePasswordLine(void)
 	}
 }
 
+/* ========================================================================= */
+/*
+ * Send SMS code request into ISDS
+ */
+void DlgChangePwd::sendSmsCode(void)
+/* ========================================================================= */
+{
+	Q_ASSERT(!m_userName.isEmpty());
+
+	/* show Premium SMS request dialog */
+	QMessageBox::StandardButton reply = QMessageBox::question(this,
+	    tr("SMS code for account ") +
+	    AccountModel::globAccounts[m_userName].accountName(),
+	    tr("Account \"%1\" requires authentication via security code "
+	    "for connection to databox.")
+	        .arg(AccountModel::globAccounts[m_userName].accountName())
+	    + "<br/>" +
+	    tr("Security code will be sent you via Premium SMS.") +
+	    "<br/><br/>" +
+	    tr("Do you want to send Premium SMS with "
+	    "security code into your mobile phone?"),
+	    QMessageBox::Yes | QMessageBox::No, QMessageBox::Yes);
+
+	if (reply == QMessageBox::No) {
+		return;
+	}
+
+	struct isds_ctx *session = isdsSessions.session(m_userName);
+	if (NULL == session) {
+		Q_ASSERT(0);
+		return;
+	}
+
+	isds_error status;
+	char * refnumber = NULL;
+	struct isds_otp *otp = NULL;
+	otp = (struct isds_otp *) malloc(sizeof(struct isds_otp));
+	memset(otp, 0, sizeof(struct isds_otp));
+	otp->method = OTP_TIME;
+	otp->otp_code = NULL;
+
+	status = isds_change_password(session,
+	    this->currentPwdLineEdit->text().toUtf8().constData(),
+	    this->newPwdLineEdit->text().toUtf8().constData(),
+	    otp, &refnumber);
+
+	free(refnumber);
+	free(otp->otp_code);
+	free(otp);
+
+	if (IE_PARTIAL_SUCCESS == status) {
+		QMessageBox::information(this, tr("Enter SMS security code"),
+		    tr("SMS security code for account \"%1\"<br/>"
+		    "has been sent on your mobile phone...")
+		    .arg(AccountModel::globAccounts[m_userName].accountName())
+		     + "<br/><br/>" +
+		    tr("Enter SMS security code for account")
+		    + "<br/><b>"
+		    + AccountModel::globAccounts[m_userName].accountName()
+		    + " </b>(" + m_userName + ").",
+		    QMessageBox::Ok);
+		this->otpLabel->setText(tr("Enter SMS code:"));
+		this->smsPushButton->setEnabled(false);
+	} else {
+		QMessageBox::critical(this, tr("Login error"),
+		    tr("An error occurred while preparing "
+		    "request for SMS with OTP security code.") +
+		    "<br/><br/>" +
+		    tr("Please try again later or you have to use the "
+		    "official web interface of Datové schránky for "
+		    "access to your data box."),
+		    QMessageBox::Ok);
+	}
+}
+
 
 /* ========================================================================= */
 /*
@@ -172,14 +271,20 @@ void DlgChangePwd::changePassword(void)
 	isds_error status;
 	char * refnumber = NULL;
 
-	if (m_accountInfo.loginMethod() == LIM_HOTP ||
-	    m_accountInfo.loginMethod() == "topt") {
+	struct isds_ctx *session = isdsSessions.session(m_userName);
+	if (NULL == session) {
+		Q_ASSERT(0);
+		return;
+	}
 
+	if (AccountModel::globAccounts[m_userName].loginMethod() == LIM_HOTP ||
+	    AccountModel::globAccounts[m_userName].loginMethod() == LIM_TOTP) {
 		struct isds_otp *otp = NULL;
 		otp = (struct isds_otp *) malloc(sizeof(struct isds_otp));
 		memset(otp, 0, sizeof(struct isds_otp));
 
-		if (m_accountInfo.loginMethod() == LIM_HOTP) {
+		if (AccountModel::globAccounts[m_userName].loginMethod() ==
+		    LIM_HOTP) {
 			otp->method = OTP_HMAC;
 		} else {
 			otp->method = OTP_TIME;
@@ -189,8 +294,7 @@ void DlgChangePwd::changePassword(void)
 		    strdup(this->secCodeLineEdit->text().toUtf8().constData())
 		    : NULL;
 
-		status = isds_change_password(
-		    isdsSessions.session(m_accountInfo.userName()),
+		status = isds_change_password(session,
 		    this->currentPwdLineEdit->text().toUtf8().constData(),
 		    this->newPwdLineEdit->text().toUtf8().constData(),
 		    otp, &refnumber);
@@ -198,33 +302,42 @@ void DlgChangePwd::changePassword(void)
 		free(otp->otp_code);
 		free(otp);
 	} else {
-		status = isds_change_password(
-		    isdsSessions.session(m_accountInfo.userName()),
+		status = isds_change_password(session,
 		    this->currentPwdLineEdit->text().toUtf8().constData(),
 		    this->newPwdLineEdit->text().toUtf8().constData(),
 		    NULL, &refnumber);
 	}
 
-	QString result(refnumber);
+	free(refnumber);
 
 	if (status == IE_SUCCESS) {
 		QMessageBox::information(this, tr("Password has been changed"),
-		    tr("Password has been changed successfully...") + "\n" +
-		    tr("Reference number: ") + result,
+		    tr("Password has been changed "
+		        "successfully on the server ISDS.")
+		    + "\n\n" +
+		    tr("Restart the application. Also don't forget to remember "
+		        "the new password so you will still be able to log "
+		        "into your data box via the web interface."),
 		    QMessageBox::Ok);
 
-		AccountModel *model = dynamic_cast<AccountModel*>(
-		    m_accountList.model());
-		QModelIndex index = m_accountList.currentIndex();
-		QStandardItem *item = model->itemFromIndex(index);
-		QStandardItem *itemTop = AccountModel::itemTop(item);
-		AccountModel::SettingsMap itemSettings =
-		    itemTop->data(ROLE_ACNT_CONF_SETTINGS).toMap();
-		itemSettings.setPassword(this->newPwdLineEdit->text());
+		AccountModel::globAccounts[m_userName].setPassword(
+		    this->newPwdLineEdit->text());
+
+		/* TODO - delete and create new
+		 * isds context with new settings
+		 */
 	} else {
+		Q_ASSERT(!m_userName.isEmpty());
+		QString error = tr("Error: ") + isds_strerror(status);
+		QString isdslog = isds_long_message(session);
+		if (!isdslog.isEmpty()) {
+			error = tr("ISDS returns: ") + isdslog;
+		}
+
 		QMessageBox::warning(this, tr("Password error"),
-		    tr("An error occurred while password was changed")
-		    + "\n" + tr("ErrorType: ") + isds_strerror(status),
+		    tr("An error occurred while password was changed.")
+		    + "\n\n" + error + "\n\n" +
+		    tr("You have to fix the problem and try to again."),
 		    QMessageBox::Ok);
 	}
 }
@@ -232,9 +345,12 @@ void DlgChangePwd::changePassword(void)
 
 /* ========================================================================= */
 /*
- * Set of possilbe char to generation fo passsword
+ * Set of possible chars for generation of new password
  */
 const QString DlgChangePwd::possibleCharacters(
-    "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789");
-const int DlgChangePwd::randomStringLength = 10;
+    "ABCDEFGHIJKLMNOPQRSTUVWXYZ"
+    "abcdefghijklmnopqrstuvwxyz"
+    "0123456789"
+    "!#$%&()*+,-.:=?@[]_{|}~");
+const int DlgChangePwd::randomStringLength = PWD_MIN_LENGTH;
 /* ========================================================================= */
