@@ -27,11 +27,12 @@
 
 #include "dlg_ds_search.h"
 #include "src/io/isds_sessions.h"
+#include "src/views/table_home_end_filter.h"
 
 
 DlgDsSearch::DlgDsSearch(Action action, QTableWidget *recipientTableWidget,
-    QString dbType, bool dbEffectiveOVM, bool dbOpenAddressing,
-    QWidget *parent, QString userName)
+    const QString &dbType, bool dbEffectiveOVM, bool dbOpenAddressing,
+    QWidget *parent, const QString &userName)
     : QDialog(parent),
     m_action(action),
     m_recipientTableWidget(recipientTableWidget),
@@ -128,6 +129,9 @@ void DlgDsSearch::initSearchWindow(void)
 	this->resultsTableWidget->
 	    setEditTriggers(QAbstractItemView::NoEditTriggers);
 
+	this->resultsTableWidget->installEventFilter(
+	    new TableHomeEndFilter(this));
+
 	pingTimer = new QTimer(this);
 	pingTimer->start(DLG_ISDS_KEEPALIVE_MS);
 
@@ -289,6 +293,7 @@ void DlgDsSearch::searchDataBox(void)
 		return;
 	}
 
+	struct isds_ctx *session;
 	struct isds_PersonName *personName = NULL;
 	struct isds_Address *address = NULL;
 	struct isds_DbOwnerInfo *ownerInfo = NULL;
@@ -337,6 +342,12 @@ void DlgDsSearch::searchDataBox(void)
 	status = isdsSearch(&boxes, m_userName, ownerInfo);
 	isds_DbOwnerInfo_free(&ownerInfo);
 
+	session = isdsSessions.session(m_userName);
+	if (NULL == session) {
+		Q_ASSERT(0);
+		goto fail;
+	}
+
 	switch (status) {
 	case IE_SUCCESS:
 		break;
@@ -345,8 +356,7 @@ void DlgDsSearch::searchDataBox(void)
 	case IE_INVAL:
 	case IE_INVALID_CONTEXT:
 		QMessageBox::information(this, tr("Search result"),
-		    isdsLongMessage(isdsSessions.session(m_userName)),
-		    QMessageBox::Ok);
+		    isdsLongMessage(session), QMessageBox::Ok);
 		goto fail;
 		break;
 	case IE_ISDS:
@@ -359,8 +369,7 @@ void DlgDsSearch::searchDataBox(void)
 	case IE_XML:
 		QMessageBox::information(this, tr("Search result"),
 		    tr("It is not possible find databox because") + ":\n\n" +
-		    isdsLongMessage(isdsSessions.session(m_userName)),
-		    QMessageBox::Ok);
+		    isdsLongMessage(session), QMessageBox::Ok);
 		goto fail;
 		break;
 	default:
@@ -564,9 +573,9 @@ void DlgDsSearch::insertDsItems(void)
 			    item(i,3)->text() + " " +
 			    this->resultsTableWidget->item(i,4)->text());
 			this->m_recipientTableWidget->setItem(row,2,item);
-			item = new QTableWidgetItem;
-			item->setText(this->resultsTableWidget->
-			    item(i,5)->text());
+			item = new QTableWidgetItem;			
+			item->setText(m_dbEffectiveOVM ? tr("no") :
+			    this->resultsTableWidget->item(i,5)->text());
 			item->setTextAlignment(Qt::AlignCenter);
 			this->m_recipientTableWidget->setItem(row,3,item);
 		}
