@@ -24,13 +24,14 @@
 #ifndef _DLG_SEND_MESSAGE_H_
 #define _DLG_SEND_MESSAGE_H_
 
+#include <QDateTime>
 #include <QDialog>
 #include <QFileDialog>
 #include <QTimer>
 #include <QTreeView>
 
 #include "src/common.h"
-#include "src/io/message_db.h"
+#include "src/io/message_db_set.h"
 #include "ui_dlg_send_message.h"
 #include "src/io/isds_sessions.h"
 #include "src/models/accounts_model.h"
@@ -46,23 +47,28 @@ public:
 		ACT_NEW_FROM_TMP
 	};
 
-	class sendMsgResultStruct {
+	class MsgSendingResult {
 	public:
+		int status;
 		QString dbID;
 		QString recipientName;
-		QString dmID;
+		qint64 dmId;
 		bool isPDZ;
-		int status;
 		QString errInfo;
+
+		MsgSendingResult(void)
+		    : status(-1), dbID(), recipientName(), dmId(-1),
+		    isPDZ(false), errInfo()
+		{ }
 	};
 
-	DlgSendMessage(MessageDb &messDb, QString &dbId, QString &senderName,
-	    Action action, qint64 msgId,
-	    const AccountModel::SettingsMap &accountInfo,
-	    QString dbType, bool dbEffectiveOVM, bool dbOpenAddressing,
-	    QString &lastAttAddPath, QString &pdzCredit,
-	    QWidget *parent = 0
-	    );
+	DlgSendMessage(MessageDbSet &dbSet, const QString &dbId,
+	    const QString &senderName, Action action, qint64 msgId,
+	    const QDateTime &deliveryTime,
+	    const QString &userName, const QString &dbType,
+	    bool dbEffectiveOVM, bool dbOpenAddressing,
+	    QString &lastAttAddPath, const QString &pdzCredit,
+	    QWidget *parent = 0);
 
 private slots:
 	void on_cancelButton_clicked(void);
@@ -81,31 +87,39 @@ private slots:
 	void sendMessage(void);
 	void pingIsdsServer(void);
 	void addDbIdToRecipientList(void);
+	void tableItemDoubleClicked(QTableWidgetItem *item);
 
 private:
 	QTimer *pingTimer;
-	void initNewMessageDialog(void);
 	qint64 m_msgID;
+	QDateTime m_deliveryTime;
 	const QString m_dbId;
 	const QString m_senderName;
 	const Action m_action;
-	AccountModel::SettingsMap m_accountInfo;
-	QString m_dbType;
+	const QString m_userName;
+	const QString m_dbType;
 	bool m_dbEffectiveOVM;
 	bool m_dbOpenAddressing;
 	QString &m_lastAttAddPath;
-	QString &m_pdzCredit;
-	MessageDb &m_messDb;
+	const QString m_pdzCredit;
+	MessageDbSet &m_dbSet;
 	int m_attachSize;
 	QString m_dmType;
 	QString m_dmSenderRefNumber;
 
+	void initNewMessageDialog(void);
 	int cmptAttachmentSize(void);
 	void fillDlgAsReply(void);
 	void fillDlgFromTmpMsg(void);
 	int showInfoAboutPDZ(int pdzCnt);
-	QString getFileBase64(QString filePath);
-	int getFileSizeFromBase64(QString fileBase64);
+
+	struct isds_list *buildDocuments(void) const;
+	struct isds_envelope *buildEnvelope(void) const;
+	MsgSendingResult sendSingleMessage(struct isds_message *message,
+	    int row) const;
+
+	static
+	QByteArray getFileBase64(const QString &filePath);
 	QString getUserInfoFormIsds(QString idDbox);
 };
 
