@@ -22,7 +22,6 @@
  */
 
 
-#include <QAbstractTableModel>
 #include <QDateTime>
 #include <QDesktopServices>
 #include <QDialog>
@@ -30,7 +29,6 @@
 #include <QFileDialog>
 #include <QMenu>
 #include <QMessageBox>
-#include <QTemporaryFile>
 #include <QTimeZone>
 #include <QUrl>
 
@@ -39,193 +37,41 @@
 #include "src/gui/dlg_signature_detail.h"
 #include "src/gui/dlg_view_zfo.h"
 #include "src/io/dbs.h"
+#include "src/log/log.h"
 #include "src/gui/dlg_import_zfo.h"
-
-#define COL_NUM 2
-#define FNAME_COL 0
-#define FSIZE_COL 1
-
-
-const QVector<QString> AttachmentModel::m_headerLabels = {
-QObject::tr("File name"),
-QObject::tr("Size")
-};
+#include "src/views/table_home_end_filter.h"
 
 
 /* ========================================================================= */
 /*
  * Constructor.
  */
-AttachmentModel::AttachmentModel(QObject *parent)
-/* ========================================================================= */
-    : QAbstractTableModel(parent),
-    m_docs()
-{
-}
-
-
-/* ========================================================================= */
-/*
- * Destructor.
- */
-AttachmentModel::~AttachmentModel(void)
-/* ========================================================================= */
-{
-}
-
-
-/* ========================================================================= */
-/*
- * Returns row count.
- */
-int AttachmentModel::rowCount(const QModelIndex &parent) const
-/* ========================================================================= */
-{
-	/* unused */
-	(void) parent;
-
-	return m_docs.size();
-}
-
-
-/* ========================================================================= */
-/*
- * Returns column count.
- */
-int AttachmentModel::columnCount(const QModelIndex &parent) const
-/* ========================================================================= */
-{
-	/* unused */
-	(void) parent;
-
-	return COL_NUM;
-}
-
-
-/* ========================================================================= */
-/*
- * Returns data.
- */
-QVariant AttachmentModel::data(const QModelIndex &index, int role) const
-/* ========================================================================= */
-{
-	int row, col;
-
-	switch (role) {
-	case Qt::DisplayRole:
-		row = index.row();
-		col = index.column();
-		Q_ASSERT(row < m_docs.size());
-		Q_ASSERT(col < COL_NUM);
-
-		if (FNAME_COL == col) {
-			/* File name. */
-			return QString(m_docs[row]->dmFileDescr);
-		} else if (FSIZE_COL == col) {
-			/* File size. */
-			return QString::number(m_docs[row]->data_length);
-		}
-
-		return QVariant();
-		break;
-	case Qt::TextAlignmentRole:
-		return Qt::AlignLeft;
-		break;
-	default:
-		return QVariant();
-		break;
-	}
-}
-
-
-/* ========================================================================= */
-/*
- * Returns header data.
- */
-QVariant AttachmentModel::headerData(int section,
-    Qt::Orientation orientation, int role) const
-/* ========================================================================= */
-{
-	(void) orientation; /* Unused. */
-
-	if (role != Qt::DisplayRole) {
-		return QVariant();
-	}
-	return m_headerLabels[section];
-}
-
-
-/* ========================================================================= */
-/*
- * Set attachment model according to message content.
- */
-bool AttachmentModel::setModelData(const isds_message *message)
-/* ========================================================================= */
-{
-	const struct isds_list *docListItem;
-
-	if (NULL == message) {
-		Q_ASSERT(0);
-		return false;
-	}
-
-	docListItem = message->documents;
-	if (NULL == docListItem) {
-		Q_ASSERT(0);
-		return false;
-	}
-
-	this->beginResetModel();
-
-	m_docs.clear();
-
-	while (NULL != docListItem) {
-		Q_ASSERT(NULL != docListItem->data);
-		m_docs.append((struct isds_document *) docListItem->data);
-		docListItem = docListItem->next;
-	}
-
-	this->endResetModel();
-
-	return true;
-}
-
-
-/* ========================================================================= */
-/*
- * Get attachment content.
- */
-QByteArray AttachmentModel::attachmentData(int indexRow) const
-/* ========================================================================= */
-{
-	Q_ASSERT(m_docs.size() > 0);
-
-	QByteArray data((char *) m_docs[indexRow]->data,
-	    (int) m_docs[indexRow]->data_length);
-
-	return data;
-}
-
-
-/* ========================================================================= */
-/*
- * Constructor.
- */
-DlgViewZfo::DlgViewZfo(const isds_message *isdsMsg, int zfoType,
-    QWidget *parent)
+DlgViewZfo::DlgViewZfo(const QString &zfoFileName, QWidget *parent)
 /* ========================================================================= */
     : QDialog(parent),
-    m_message(isdsMsg),
-    m_zfoType(zfoType),
+    m_message(NULL),
     m_attachmentModel(0)
 {
 	setupUi(this);
 
-	Q_ASSERT(NULL != m_message);
+	/* TODO -- Load message ZFO. */
+	parseZfoFile(zfoFileName);
+
+	if(NULL == m_message) {
+		/* Just show error message. */
+		this->attachmentTable->hide();
+		envelopeTextEdit->setHtml(
+		    "<h3>" + tr("Error parsing content") + "</h3><br/>" +
+		    tr("Cannot parse the content of file '%1'.")
+		        .arg(zfoFileName));
+		envelopeTextEdit->setReadOnly(true);
+		signaturePushButton->setEnabled(false);
+		return;
+	}
 
 	/* TODO -- Adjust splitter sizes. */
 
-	if (m_zfoType == ImportZFODialog::IMPORT_DELIVERY_ZFO) {
+	if (ImportZFODialog::IMPORT_DELIVERY_ZFO == m_zfoType) {
 		this->attachmentTable->hide();
 		envelopeTextEdit->setHtml(
 		    deliveryDescriptionHtml(
@@ -237,7 +83,7 @@ DlgViewZfo::DlgViewZfo(const isds_message *isdsMsg, int zfoType,
 	} else {
 		this->attachmentTable->setEnabled(true);
 		this->attachmentTable->show();
-		m_attachmentModel.setModelData(isdsMsg);
+		m_attachmentModel.setModelData(m_message);
 		envelopeTextEdit->setHtml(
 		    messageDescriptionHtml(m_attachmentModel.rowCount(),
 		        m_message->raw, m_message->raw_length,
@@ -255,6 +101,7 @@ DlgViewZfo::DlgViewZfo(const isds_message *isdsMsg, int zfoType,
 		connect(attachmentTable, SIGNAL(doubleClicked(QModelIndex)),
 		    this, SLOT(attachmentItemDoubleClicked(QModelIndex)));
 
+		attachmentTable->installEventFilter(new TableHomeEndFilter(this));
 	}
 
 	/* Signature details. */
@@ -263,6 +110,19 @@ DlgViewZfo::DlgViewZfo(const isds_message *isdsMsg, int zfoType,
 }
 
 
+/* ========================================================================= */
+/*
+ * Destructor.
+ */
+DlgViewZfo::~DlgViewZfo(void)
+/* ========================================================================= */
+{
+	if (NULL != m_message) {
+		isds_message_free(&m_message);
+	}
+}
+
+
 /* ========================================================================= */
 /*
  * Generates menu to selected message item.
@@ -313,6 +173,40 @@ void DlgViewZfo::attachmentItemDoubleClicked(const QModelIndex &index)
 }
 
 
+/* ========================================================================= */
+/*
+ * Loads ZFO file.
+ */
+void DlgViewZfo::parseZfoFile(const QString &zfoFileName)
+/* ========================================================================= */
+{
+	/* Logging purposes. */
+	struct isds_ctx *dummy_session = isds_ctx_create();
+	if (NULL == dummy_session) {
+		logError("%s\n", "Cannot create dummy ISDS session.");
+		goto fail;
+	}
+
+	m_zfoType = ImportZFODialog::IMPORT_MESSAGE_ZFO;
+	Q_ASSERT(NULL == m_message);
+	m_message = loadZfoFile(dummy_session, zfoFileName, m_zfoType);
+	if (NULL == m_message) {
+		m_zfoType = ImportZFODialog::IMPORT_DELIVERY_ZFO;
+		m_message = loadZfoFile(dummy_session, zfoFileName, m_zfoType);
+		if (NULL == m_message) {
+			logError("Cannot parse file '%s'.\n",
+			    zfoFileName.toUtf8().constData());
+			goto fail;
+		}
+	}
+
+fail:
+	if (NULL != dummy_session) {
+		isds_ctx_free(&dummy_session);
+	}
+}
+
+
 /* ========================================================================= */
 /*
  * Saves selected attachment to file.
@@ -443,6 +337,8 @@ void DlgViewZfo::openSelectedAttachment(void)
 		Q_ASSERT(0);
 		return;
 	}
+	attachName.replace(QRegExp("\\s"), "_").replace(
+	    QRegExp("[^a-zA-Z\\d\\.\\-_]"), "x");
 	/* TODO -- Add message id into file name? */
 	QString fileName = TMP_ATTACHMENT_PREFIX + attachName;
 
@@ -485,95 +381,31 @@ void DlgViewZfo::showSignatureDetails(void)
  * Generate description from supplied message.
  */
 QString DlgViewZfo::messageDescriptionHtml(int attachmentCount,
-    const void *msgDER, size_t msgSize, const void *tstDER, size_t tstSize)
+    const void *msgDER, size_t msgSize, const void *tstDER,
+    size_t tstSize) const
 /* ========================================================================= */
 {
-	const isds_envelope *envelope;
-	isds_message_status *state;
+	if (NULL == m_message) {
+		Q_ASSERT(0);
+		return QString();
+	}
 
-	envelope = m_message->envelope;
-	Q_ASSERT(NULL != envelope);
+	const isds_envelope *envelope = m_message->envelope;
+	if (NULL == envelope) {
+		Q_ASSERT(0);
+		return QString();
+	}
 
-	state = envelope->dmMessageStatus;
-	Q_ASSERT(NULL != state);
+	QString html(indentDivStart);
 
-	QString html;
+	envelopeHeaderDescriptionHtml(html, envelope);
 
-	html += indentDivStart;
-	html += "<h3>" + tr("Identification") + "</h3>";
-
-	html += strongAccountInfoLine(tr("ID"), envelope->dmID);
-	html += strongAccountInfoLine(tr("Subject"), envelope->dmAnnotation);
-	html += strongAccountInfoLine(tr("Message type"), envelope->dmType);
-
-	html += "<br/>";
-
-	/* Information about message author. */
-	html += strongAccountInfoLine(tr("Sender"), envelope->dmSender);
-	html += strongAccountInfoLine(tr("Sender Databox ID"), envelope->dbIDSender);
-	html += strongAccountInfoLine(tr("Sender Address"),
-	    envelope->dmSenderAddress);
-
-	html += "<br/>";
-
-	html += strongAccountInfoLine(tr("Recipient"), envelope->dmRecipient);
-	html += strongAccountInfoLine(tr("Recipient Databox ID"), envelope->dbIDRecipient);
-	html += strongAccountInfoLine(tr("Recipient Address"),
-	    envelope->dmRecipientAddress);
-
-	html += "<h3>" + tr("Status") + "</h3>";
-
-	html += strongAccountInfoLine(tr("Delivery time"),
-	    dateTimeStrFromDbFormat(
-	        timevalToDbFormat(envelope->dmDeliveryTime),
-	        dateTimeDisplayFormat));
-	html += strongAccountInfoLine(tr("Acceptance time"),
-	    dateTimeStrFromDbFormat(
-	        timevalToDbFormat(envelope->dmAcceptanceTime),
-	        dateTimeDisplayFormat));
-	html += strongAccountInfoLine(tr("Status"),
-	    QString::number(convertHexToDecIndex(
-	        *(envelope->dmMessageStatus))) + " -- " +
-	    msgStatusToText(convertHexToDecIndex(
-	        *(envelope->dmMessageStatus))));
 	html += strongAccountInfoLine(tr("Attachments"),
 	    QString::number(attachmentCount));
 
-	html += "<h3>" + tr("Signature") + "</h3>";
+	signatureFooterDescription(html, msgDER, msgSize, tstDER, tstSize);
 
-	QString resultStr;
-	if (1 == raw_msg_verify_signature(msgDER, msgSize, 0, 0)) {
-		resultStr = QObject::tr("Valid");
-	} else {
-		resultStr = QObject::tr("Invalid")  + " -- " +
-		    QObject::tr("Message signature and content do not "
-		        "correspond!");
-	}
-	html += strongAccountInfoLine(tr("Message signature"), resultStr);
-	if (1 == raw_msg_verify_signature_date(msgDER, msgSize,
-	        QDateTime::currentDateTime().toTime_t(), 0)) {
-		resultStr = QObject::tr("Valid");
-	} else {
-		resultStr = QObject::tr("Invalid");
-	}
-	if (!globPref.check_crl) {
-		resultStr += " (" +
-		    QObject::tr("Certificate revocation check is turned off!") +
-		    ")";
-	}
-	html += strongAccountInfoLine(tr("Signing certificate"), resultStr);
-	time_t utc_time = 0;
-	QDateTime tst;
-	int ret = raw_tst_verify(tstDER, tstSize, &utc_time);
-	if (-1 != ret) {
-		tst = QDateTime::fromTime_t(utc_time);
-	}
-	resultStr = (1 == ret) ? QObject::tr("Valid") : QObject::tr("Invalid");
-	if (-1 != ret) {
-		resultStr += " (" + tst.toString("dd.MM.yyyy hh:mm:ss") + " " +
-		    tst.timeZone().abbreviation(tst) + ")";
-	}
-	html += strongAccountInfoLine(tr("Time stamp"), resultStr);
+	html += divEnd;
 
 	return html;
 }
@@ -585,25 +417,55 @@ QString DlgViewZfo::messageDescriptionHtml(int attachmentCount,
  * Generate description from supplied delivery info.
  */
 QString DlgViewZfo::deliveryDescriptionHtml(const void *msgDER,
-    size_t msgSize, const void *tstDER, size_t tstSize)
+    size_t msgSize, const void *tstDER, size_t tstSize) const
 /* ========================================================================= */
 {
-	const isds_envelope *envelope;
-	const struct isds_list *event;
-	isds_message_status *state;
+	if (NULL == m_message) {
+		Q_ASSERT(0);
+		return QString();
+	}
 
-	envelope = m_message->envelope;
-	Q_ASSERT(NULL != envelope);
+	const isds_envelope *envelope = m_message->envelope;
+	if (NULL == envelope) {
+		Q_ASSERT(0);
+		return QString();
+	}
 
-	event = m_message->envelope->events;
-	Q_ASSERT(NULL != event);
+	QString html(indentDivStart);
 
-	state = envelope->dmMessageStatus;
-	Q_ASSERT(NULL != state);
+	envelopeHeaderDescriptionHtml(html, envelope);
 
-	QString html;
+	html += strongAccountInfoLine(tr("Events"),"");
 
 	html += indentDivStart;
+	const struct isds_list *event = envelope->events;
+	while (NULL != event) {
+		isds_event *item = (isds_event *) event->data;
+		html += strongAccountInfoLine(
+		        dateTimeStrFromDbFormat(timevalToDbFormat(item->time),
+		        dateTimeDisplayFormat),
+		        item->description);
+		event = event->next;
+	}
+	html += divEnd;
+
+	signatureFooterDescription(html, msgDER, msgSize, tstDER, tstSize);
+
+	html += divEnd;
+
+	return html;
+}
+
+/* ========================================================================= */
+bool DlgViewZfo::envelopeHeaderDescriptionHtml(QString &html,
+    const isds_envelope *envelope)
+/* ========================================================================= */
+{
+	if (NULL == envelope) {
+		Q_ASSERT(0);
+		return false;
+	}
+
 	html += "<h3>" + tr("Identification") + "</h3>";
 
 	html += strongAccountInfoLine(tr("ID"), envelope->dmID);
@@ -635,24 +497,24 @@ QString DlgViewZfo::deliveryDescriptionHtml(const void *msgDER,
 	    dateTimeStrFromDbFormat(
 	        timevalToDbFormat(envelope->dmAcceptanceTime),
 	        dateTimeDisplayFormat));
-	html += strongAccountInfoLine(tr("Status"),
-	    QString::number(convertHexToDecIndex(
-	        *(envelope->dmMessageStatus))) + " -- " +
-	    msgStatusToText(convertHexToDecIndex(
-	        *(envelope->dmMessageStatus))));
-	html += strongAccountInfoLine(tr("Events"),"");
 
-	html += indentDivStart;
-	while (0 != event) {
-		isds_event *item = (isds_event *) event->data;
-		html += strongAccountInfoLine(
-		        dateTimeStrFromDbFormat(timevalToDbFormat(item->time),
-		        dateTimeDisplayFormat),
-		        item->description);
-		event = event->next;
+	QString statusString;
+	if (NULL != envelope->dmMessageStatus) {
+		statusString =
+		    QString::number(convertHexToDecIndex(*(envelope->dmMessageStatus))) +
+		    " -- " +
+		    msgStatusToText(convertHexToDecIndex(*(envelope->dmMessageStatus)));
 	}
-	html += divEnd;
+	html += strongAccountInfoLine(tr("Status"), statusString);
 
+	return true;
+}
+
+/* ========================================================================= */
+bool DlgViewZfo::signatureFooterDescription(QString &html,
+    const void *msgDER, size_t msgSize, const void *tstDER, size_t tstSize)
+/* ========================================================================= */
+{
 	html += "<h3>" + tr("Signature") + "</h3>";
 
 	QString resultStr;
@@ -689,7 +551,7 @@ QString DlgViewZfo::deliveryDescriptionHtml(const void *msgDER,
 	}
 	html += strongAccountInfoLine(tr("Time stamp"), resultStr);
 
-	return html;
+	return true;
 }
 
 /* ========================================================================= */
