@@ -38,6 +38,81 @@
 GlobIsdsSessions isdsSessions;
 
 
+/* ========================================================================= */
+/*!
+ * @brief Logging facility name.
+ */
+static
+const char * logFacilityName(isds_log_facility facility)
+/* ========================================================================= */
+{
+	const char *str = "libisds unknown";
+
+	switch (facility) {
+	case ILF_NONE:
+		str = "libisds none";
+		break;
+	case ILF_HTTP:
+		str = "libisds http";
+		break;
+	case ILF_SOAP:
+		str = "libisds soap";
+		break;
+	case ILF_ISDS:
+		str = "libisds isds";
+		break;
+	case ILF_FILE:
+		str = "libisds file";
+		break;
+	case ILF_SEC:
+		str = "libisds sec";
+		break;
+	case ILF_XML:
+		str = "libisds xml";
+		break;
+	default:
+		break;
+	}
+
+	return str;
+}
+
+
+/* ========================================================================= */
+/*!
+ * @brief Logging callback used in libisds.
+ */
+static
+void logCallback(isds_log_facility facility, isds_log_level level,
+    const char *message, int length, void *data)
+/* ========================================================================= */
+{
+	(void) data;
+	(void) length;
+	const char *logFac = logFacilityName(facility);
+
+	switch (level) {
+	case ILL_NONE:
+	case ILL_CRIT:
+	case ILL_ERR:
+		logErrorMl("%s: %s", logFac, message);
+		break;
+	case ILL_WARNING:
+		logWarningMl("%s: %s", logFac, message);
+		break;
+	case ILL_INFO:
+		logInfoMl("%s: %s", logFac, message);
+		break;
+	case ILL_DEBUG:
+		logDebugMlLv3("%s: %s", logFac, message);
+		break;
+	default:
+		logError("Unknown ISDS log level %d.\n", level);
+		break;
+	}
+}
+
+
 /* ========================================================================= */
 GlobIsdsSessions::GlobIsdsSessions(void)
 /* ========================================================================= */
@@ -49,12 +124,14 @@ GlobIsdsSessions::GlobIsdsSessions(void)
 	status = isds_init();
 	if (IE_SUCCESS != status) {
 		Q_ASSERT(0);
-		qWarning() << "Unsuccessful ISDS initialisation.";
+		logError("%s\n", "Unsuccessful ISDS initialisation.");
 		/* TODO -- What to do on failure? */
 	}
 
+	isds_set_log_callback(logCallback, NULL);
+
 	/* Logging. */
-	//isds_set_logging(ILF_ALL, ILL_ALL);
+	isds_set_logging(ILF_ALL, ILL_ALL);
 }
 
 
@@ -71,18 +148,18 @@ GlobIsdsSessions::~GlobIsdsSessions(void)
 
 		status = isds_logout(isdsSession);
 		if (IE_SUCCESS != status) {
-			qWarning() << "Error in ISDS logout procedure.";
+			logWarning("%s\n", "Error in ISDS logout procedure.");
 		}
 
 		status = isds_ctx_free(&isdsSession);
 		if (IE_SUCCESS != status) {
-			qDebug() << "Error freeing ISDS session.";
+			logWarning("%s\n", "Error freeing ISDS session.");
 		}
 	}
 
 	status = isds_cleanup();
 	if (IE_SUCCESS != status) {
-		qDebug() << "Unsuccessful ISDS clean-up.";
+		logWarning("%s\n", "Unsuccessful ISDS clean-up.");
 	}
 }
 
@@ -135,13 +212,15 @@ struct isds_ctx * GlobIsdsSessions::createCleanSession(const QString &userName,
 
 	isds_session = isds_ctx_create();
 	if (NULL == isds_session) {
-		qDebug() << "Error creating ISDS session.";
+		logError("Error creating ISDS session for user '%s'.\n",
+		    userName.toUtf8().constData());
 		goto fail;
 	}
 
 	status = isds_set_timeout(isds_session, connectionTimeoutMs);
 	if (IE_SUCCESS != status) {
-		qDebug() << "Error setting time-out.";
+		logError("Error setting time-out for user '%s'.\n",
+		    userName.toUtf8().constData());
 		goto fail;
 	}
 
@@ -152,7 +231,9 @@ fail:
 	if (NULL != isds_session) {
 		status = isds_ctx_free(&isds_session);
 		if (IE_SUCCESS != status) {
-			qDebug() << "Error freeing ISDS session.";
+			logWarning(
+			    "Error freeing ISDS session for user '%s'.\n",
+			    userName.toUtf8().constData());
 		}
 	}
 	return 0;
@@ -179,7 +260,8 @@ bool GlobIsdsSessions::setSessionTimeout(const QString &userName,
 
 	status = isds_set_timeout(isds_session, timeoutMs);
 	if (IE_SUCCESS != status) {
-		logError("%s\n", "Error setting time-out.");
+		logError("Error setting time-out for user '%s'.\n",
+		    userName.toUtf8().constData());
 		return false;
 	}
 
@@ -194,8 +276,6 @@ bool GlobIsdsSessions::setSessionTimeout(const QString &userName,
 struct isds_ctx * GlobIsdsSessions::session(const QString &userName) const
 /* ========================================================================= */
 {
-	Q_ASSERT(NULL != m_sessions.value(userName, NULL));
-
 	return m_sessions.value(userName, NULL);
 }
 
@@ -233,7 +313,6 @@ isds_error isdsLoginSystemCert(struct isds_ctx *isdsSession,
 
 	isds_error status = IE_ERROR;
 
-
 	if (certPath.isNull() || certPath.isEmpty()) {
 		return status;
 	}
@@ -765,8 +844,13 @@ isds_error isdsSearch(struct isds_list **result, const QString &userName,
 		return IE_ERROR;
 	}
 
-	ret = isds_FindDataBox(isdsSessions.session(userName), ownerInfo,
-	    result);
+	struct isds_ctx *session = isdsSessions.session(userName);
+	if (NULL == session) {
+		Q_ASSERT(0);
+		return IE_ERROR;
+	}
+
+	ret = isds_FindDataBox(session, ownerInfo, result);
 
 	qDebug() << ret << isds_strerror(ret);
 	return ret;
