@@ -25,115 +25,13 @@
 #include <QtAlgorithms> /* qSort() */
 #include <QDebug>
 #include <QRegExp>
+#include <QSet>
 
 #include "accounts_model.h"
 #include "src/common.h"
 #include "src/log/log.h"
 
 
-/* ========================================================================= */
-AccountModel::SettingsMap::SettingsMap(void)
-/* ========================================================================= */
-    : QMap<QString, QVariant>()
-{
-}
-
-
-/* ========================================================================= */
-AccountModel::SettingsMap::SettingsMap(const QMap<QString, QVariant> &map)
-/* ========================================================================= */
-    : QMap<QString, QVariant>(map)
-{
-}
-
-
-/* ========================================================================= */
-void AccountModel::SettingsMap::setDbDir(const QString &path)
-/* ========================================================================= */
-{
-	if (path == globPref.confDir()) {
-		/* Default path is empty. */
-		QMap<QString, QVariant>::operator[](DB_DIR) = QString();
-	} else {
-		QMap<QString, QVariant>::operator[](DB_DIR) = path;
-	}
-}
-
-
-/* ========================================================================= */
-/*
- * Empty account model constructor.
- */
-AccountModel::AccountModel(QObject *parent)
-/* ========================================================================= */
-    : QStandardItemModel(parent)
-{
-	/* Set header. */
-	this->setHorizontalHeaderItem(0, new QStandardItem(QObject::tr("Accounts")));
-}
-
-
-/* ========================================================================= */
-/*
- * Compute viewed data.
- */
-QVariant AccountModel::data(const QModelIndex &index, int role) const
-/* ========================================================================= */
-{
-	QVariant storedData;
-	switch (role) {
-	case Qt::DisplayRole:
-		storedData = QStandardItemModel::data(index,
-		    ROLE_ACNT_UNREAD_MSGS);
-		if (storedData.isValid()) {
-			QString retStr = QStandardItemModel::data(index,
-			    role).toString() + " (" +
-			    QString::number(storedData.toInt()) + ")";
-			return retStr;
-		}
-
-		return QStandardItemModel::data(index, role);
-		break;
-
-#if 0
-	case Qt::DecorationRole:
-		if (nodeAccountTop == nodeType(index)) {
-			SettingsMap settingsMap =
-			    index.data(ROLE_ACNT_CONF_SETTINGS).toMap();
-			qDebug() << "A001" << globPref.confDir();
-			if (!settingsMap.dbDir().isEmpty()) {
-				return QIcon(ICON_16x16_PATH "grey.png");
-			}
-		}
-		return QStandardItemModel::data(index, role);
-		break;
-#endif
-
-	case Qt::FontRole:
-		if (nodeAccountTop == nodeType(index)) {
-			QFont retFont;
-			retFont.setBold(true);
-			return retFont;
-		}
-
-		storedData = QStandardItemModel::data(index,
-		    ROLE_ACNT_UNREAD_MSGS);
-		if (storedData.isValid()) {
-			QFont retFont;
-			retFont.setBold(true);
-			return retFont;
-		}
-
-		return QStandardItemModel::data(index, role);
-		break;
-
-	default:
-		return QStandardItemModel::data(index, role);
-		break;
-	}
-}
-
-
 /* ========================================================================= */
 /*!
  * @brief Used for sorting credentials.
@@ -177,10 +75,36 @@ bool credentialsLessThan(const QString &s1, const QString &s2)
 
 
 /* ========================================================================= */
-/*
- * Load data from supplied settings.
- */
-void AccountModel::loadFromSettings(const QSettings &settings)
+AccountModel::SettingsMap::SettingsMap(void)
+/* ========================================================================= */
+    : QMap<QString, QVariant>()
+{
+}
+
+
+/* ========================================================================= */
+AccountModel::SettingsMap::SettingsMap(const QMap<QString, QVariant> &map)
+/* ========================================================================= */
+    : QMap<QString, QVariant>(map)
+{
+}
+
+
+/* ========================================================================= */
+void AccountModel::SettingsMap::setDbDir(const QString &path)
+/* ========================================================================= */
+{
+	if (path == globPref.confDir()) {
+		/* Default path is empty. */
+		QMap<QString, QVariant>::operator[](DB_DIR) = QString();
+	} else {
+		QMap<QString, QVariant>::operator[](DB_DIR) = path;
+	}
+}
+
+
+/* ========================================================================= */
+void AccountModel::AccountsMap::loadFromSettings(const QSettings &settings)
 /* ========================================================================= */
 {
 	debugFuncCall();
@@ -190,7 +114,7 @@ void AccountModel::loadFromSettings(const QSettings &settings)
 	SettingsMap itemSettings;
 
 	/* Clear present rows. */
-	this->removeRows(0, this->rowCount());
+	this->clear();
 
 	QStringList credetialList;
 	/* Get list of credentials. */
@@ -258,7 +182,128 @@ void AccountModel::loadFromSettings(const QSettings &settings)
 		        "").toString());
 
 		/* Associate map with item node. */
-		addAccount(itemSettings.accountName(), itemSettings);
+		Q_ASSERT(!itemSettings.userName().isEmpty());
+		this->operator[](itemSettings.userName()) = itemSettings;
+	}
+}
+
+
+AccountModel::AccountsMap AccountModel::globAccounts;
+
+
+/* ========================================================================= */
+/*
+ * Empty account model constructor.
+ */
+AccountModel::AccountModel(QObject *parent)
+/* ========================================================================= */
+    : QStandardItemModel(parent)
+{
+	/* Set header. */
+	this->setHorizontalHeaderItem(0, new QStandardItem(QObject::tr("Accounts")));
+}
+
+
+/* ========================================================================= */
+/*
+ * Compute viewed data.
+ */
+QVariant AccountModel::data(const QModelIndex &index, int role) const
+/* ========================================================================= */
+{
+	QVariant storedData;
+	switch (role) {
+	case Qt::DisplayRole:
+		storedData = QStandardItemModel::data(index,
+		    ROLE_ACNT_UNREAD_MSGS);
+		if (storedData.isValid()) {
+			QString retStr = QStandardItemModel::data(index,
+			    role).toString() + " (" +
+			    QString::number(storedData.toInt()) + ")";
+			return retStr;
+		}
+
+		return QStandardItemModel::data(index, role);
+		break;
+
+#if 0
+	case Qt::DecorationRole:
+		if (nodeAccountTop == nodeType(index)) {
+			const QString userName =
+			    index.data(ROLE_ACNT_USER_NAME).toString();
+			qDebug() << "A001" << globPref.confDir();
+			if (!globAccounts[userName].dbDir().isEmpty()) {
+				return QIcon(ICON_14x14_PATH "grey.png");
+			}
+		}
+		return QStandardItemModel::data(index, role);
+		break;
+#endif
+
+	case Qt::FontRole:
+		if (nodeAccountTop == nodeType(index)) {
+			QFont retFont;
+			retFont.setBold(true);
+			return retFont;
+		}
+
+		storedData = QStandardItemModel::data(index,
+		    ROLE_ACNT_UNREAD_MSGS);
+		if (storedData.isValid()) {
+			QFont retFont;
+			retFont.setBold(true);
+			return retFont;
+		}
+
+		return QStandardItemModel::data(index, role);
+		break;
+
+	default:
+		return QStandardItemModel::data(index, role);
+		break;
+	}
+}
+
+
+/* ========================================================================= */
+/*
+ * Load data from supplied settings.
+ */
+void AccountModel::loadFromSettings(const QSettings &settings)
+/* ========================================================================= */
+{
+	debugFuncCall();
+
+	globAccounts.loadFromSettings(settings);
+
+	QStringList groups = settings.childGroups();
+	QRegExp credRe(CREDENTIALS".*");
+
+	/* Clear present rows. */
+	this->removeRows(0, this->rowCount());
+
+	QStringList credetialList;
+	/* Get list of credentials. */
+	for (int i = 0; i < groups.size(); ++i) {
+		/* Matches regular expression. */
+		if (credRe.exactMatch(groups.at(i))) {
+			credetialList.append(groups.at(i));
+		}
+	}
+
+	/* Sort the credentials list. */
+	qSort(credetialList.begin(), credetialList.end(), credentialsLessThan);
+
+	/* For all credentials. */
+	foreach(QString group, credetialList) {
+		const QString accountName = settings.value(
+		    group + "/" + ACCOUNT_NAME,
+		    QString()).toStringList().join(", ");
+		const QString userName = settings.value(group + "/" + USER,
+		    QString()).toString();
+
+		/* Associate map with item node. */
+		addAccount(accountName, globAccounts[userName]);
 	}
 }
 
@@ -273,8 +318,9 @@ void AccountModel::saveToSettings(QSettings &settings) const
 	QString groupName;
 
 	for (int i = 0; i < this->rowCount(); ++i) {
-		const SettingsMap &itemSettings =
-		    this->item(i)->data(ROLE_ACNT_CONF_SETTINGS).toMap();
+		const QString userName =
+		    this->item(i)->data(ROLE_ACNT_USER_NAME).toString();
+		const SettingsMap &itemSettings = globAccounts[userName];
 
 		groupName = CREDENTIALS;
 		if (i > 0) {
@@ -343,7 +389,8 @@ void AccountModel::saveToSettings(QSettings &settings) const
 /*
  * Add account.
  */
-QModelIndex AccountModel::addAccount(const QString &name, const QVariant &data)
+QModelIndex AccountModel::addAccount(const QString &name,
+    const AccountModel::SettingsMap &settings)
 /* ========================================================================= */
 {
 	/* Defining a couple of items. */
@@ -361,7 +408,9 @@ QModelIndex AccountModel::addAccount(const QString &name, const QVariant &data)
 	QStandardItem *allSent = new QStandardItem(QObject::tr("Sent"));
 	allSent->setFlags(allSent->flags() & ~Qt::ItemIsEditable);
 
-	account->setData(data, ROLE_ACNT_CONF_SETTINGS);
+	Q_ASSERT(!settings.userName().isEmpty());
+	account->setData(settings.userName(), ROLE_ACNT_USER_NAME);
+	globAccounts[settings.userName()] = settings;
 
 	/* Account node is drawn bold in the data() method. */
 	account->setIcon(QIcon(ICON_3PARTY_PATH + QString("letter_16.png")));
@@ -515,6 +564,72 @@ QStandardItem * AccountModel::itemTop(QStandardItem *item)
 }
 
 
+/* ========================================================================= */
+/*
+ * Returns pointer to related top-most item.
+ */
+const QStandardItem * AccountModel::itemTopYear(const QStandardItem *item,
+    NodeType nodeType)
+/* ========================================================================= */
+{
+	item = itemTop(item);
+
+	if (0 == item) {
+		Q_ASSERT(0);
+		return 0;
+	}
+
+	if (nodeReceivedYear == nodeType) {
+		/* Get received node. */
+		item = item->child(2, 0)->child(0, 0);
+	} else if (nodeSentYear == nodeType) {
+		/* Get sent node. */
+		item = item->child(2, 0)->child(1, 0);
+	} else {
+		Q_ASSERT(0);
+		return 0;
+	}
+
+	if (0 == item) {
+		Q_ASSERT(0);
+		return 0;
+	}
+
+	return item;
+}
+
+
+/* ========================================================================= */
+QStandardItem * AccountModel::itemTopYear(QStandardItem *item, NodeType nodeType)
+/* ========================================================================= */
+{
+	item = itemTop(item);
+
+	if (0 == item) {
+		Q_ASSERT(0);
+		return 0;
+	}
+
+	if (nodeReceivedYear == nodeType) {
+		/* Get received node. */
+		item = item->child(2, 0)->child(0, 0);
+	} else if (nodeSentYear == nodeType) {
+		/* Get sent node. */
+		item = item->child(2, 0)->child(1, 0);
+	} else {
+		Q_ASSERT(0);
+		return 0;
+	}
+
+	if (0 == item) {
+		Q_ASSERT(0);
+		return 0;
+	}
+
+	return item;
+}
+
+
 /* ========================================================================= */
 /*
  * Returns index to related top-most item.
@@ -536,37 +651,6 @@ QModelIndex AccountModel::indexTop(const QModelIndex &index)
 }
 
 
-/* ========================================================================= */
-/*
- * Return related settings map.
- */
-AccountModel::SettingsMap AccountModel::settingsMap(QStandardItem *item)
-/* ========================================================================= */
-{
-	Q_ASSERT(0 != item);
-	QStandardItem *topItem = itemTop(item);
-
-	Q_ASSERT(0 != topItem);
-	return topItem->data(ROLE_ACNT_CONF_SETTINGS).toMap();
-}
-
-
-/* ========================================================================= */
-/*
- * Set settings map to related account.
- */
-void AccountModel::setSettingsMap(QStandardItem *item,
-    const AccountModel::SettingsMap &map)
-/* ========================================================================= */
-{
-	Q_ASSERT(0 != item);
-	QStandardItem *topItem = itemTop(item);
-
-	Q_ASSERT(0 != topItem);
-	topItem->setData(map, ROLE_ACNT_CONF_SETTINGS);
-}
-
-
 /* ========================================================================= */
 /*
  * Set number of unread messages in recent model nodes.
@@ -611,30 +695,13 @@ bool AccountModel::updateRecentUnread(QStandardItem *item, NodeType nodeType,
 
 /* ========================================================================= */
 /*
- * Add year node into account.
+ * Append year node into account.
  */
-bool AccountModel::addYear(QStandardItem *item, NodeType nodeType,
+bool AccountModel::appendYear(QStandardItem *item, NodeType nodeType,
     const QString &year, unsigned unreadMsgs)
 /* ========================================================================= */
 {
-	/* Find account top. */
-	item = itemTop(item);
-
-	if (0 == item) {
-		Q_ASSERT(0);
-		return false;
-	}
-
-	if (nodeReceivedYear == nodeType) {
-		/* Get received node. */
-		item = item->child(2, 0)->child(0, 0);
-	} else if (nodeSentYear == nodeType) {
-		/* Get sent node. */
-		item = item->child(2, 0)->child(1, 0);
-	} else {
-		Q_ASSERT(0);
-		return false;
-	}
+	item = itemTopYear(item, nodeType);
 
 	if (0 == item) {
 		Q_ASSERT(0);
@@ -669,31 +736,88 @@ bool AccountModel::addYear(QStandardItem *item, NodeType nodeType,
 
 /* ========================================================================= */
 /*
- * Update existing year node in account.
+ * Update year nodes.
  */
-bool AccountModel::updateYear(QStandardItem *item, NodeType nodeType,
-    const QString &year, unsigned unreadMsgs)
+bool AccountModel::updateYearNodes(QStandardItem *item, NodeType nodeType,
+    const QList< QPair<QString, int> > &yearlyUnreadList)
 /* ========================================================================= */
 {
-	/* Find account top. */
-	item = itemTop(item);
+	typedef QPair<QString, int> YearlyContent;
+
+	item = itemTopYear(item, nodeType);
 
 	if (0 == item) {
 		Q_ASSERT(0);
 		return false;
 	}
 
-	if (nodeReceivedYear == nodeType) {
-		/* Get received node. */
-		item = item->child(2, 0)->child(0, 0);
-	} else if (nodeSentYear == nodeType) {
-		/* Get sent node. */
-		item = item->child(2, 0)->child(1, 0);
-	} else {
-		Q_ASSERT(0);
-		return false;
+	QSet<QString> yearsCurrent;
+	for (int i = 0; i < item->rowCount(); ++i) {
+		yearsCurrent.insert(item->child(i, 0)->text());
 	}
 
+	QSet<QString> yearsNew;
+	foreach (const YearlyContent &content, yearlyUnreadList) {
+		yearsNew.insert(content.first);
+	}
+
+	QSet<QString> yearsRemoved = yearsCurrent - yearsNew;
+	QSet<QString> yearsAdded = yearsNew - yearsCurrent;
+
+	/* Remove unneeded nodes. */
+	int i = 0;
+	while (i < item->rowCount()) {
+		if (yearsRemoved.contains(item->child(i, 0)->text())) {
+			item->removeRow(i);
+		} else {
+			++i;
+		}
+	}
+
+	for (int i = 0; i < yearlyUnreadList.size(); ++i) {
+		const YearlyContent &content = yearlyUnreadList[i];
+		if (yearsAdded.contains(content.first)) {
+			/* Add year entry. */
+			QStandardItem *yearItem = new QStandardItem(content.first);
+			if (nodeReceivedYear == nodeType) {
+				yearItem->setIcon(QIcon(ICON_16x16_PATH +
+				    QString("datovka-message-download.png")));
+			} else {
+				yearItem->setIcon(QIcon(ICON_16x16_PATH +
+				    QString("datovka-message-reply.png")));
+			}
+			yearItem->setFlags(yearItem->flags() & ~Qt::ItemIsEditable);
+			if (0 < content.second) {
+				yearItem->setData(content.second, ROLE_ACNT_UNREAD_MSGS);
+			}
+			item->insertRow(i, yearItem);
+		} else {
+			/* Update year entry. */
+			Q_ASSERT(item->child(i, 0)->text() == content.first);
+			if (0 < content.second) {
+				item->child(i, 0)->setData(content.second,
+				    ROLE_ACNT_UNREAD_MSGS);
+			} else {
+				item->child(i, 0)->setData(QVariant(),
+				    ROLE_ACNT_UNREAD_MSGS);
+			}
+		}
+	}
+
+	return true;
+}
+
+
+/* ========================================================================= */
+/*
+ * Update existing year node in account.
+ */
+bool AccountModel::updateYear(QStandardItem *item, NodeType nodeType,
+    const QString &year, unsigned unreadMsgs)
+/* ========================================================================= */
+{
+	item = itemTopYear(item, nodeType);
+
 	if (0 == item) {
 		Q_ASSERT(0);
 		return false;
