@@ -23,6 +23,7 @@
 
 
 #include <QDebug>
+#include <QSet>
 #include <QThread>
 
 #include "worker.h"
@@ -185,34 +186,49 @@ void Worker::doJob(void)
 	int sn = 0;
 	QString errMsg;
 	qdatovka_error res = Q_SUCCESS;
+	unsigned long dmLimit = MESSAGE_LIST_LIMIT;
 
-	/* != -1 -- specifice message required. */
-	if (0 <= job.msgId) {
+	/* != -1 -- specific message required. */
+	if (0 <= job.mId.dmId) {
 
 		qDebug() << "-----------------------------------------------";
-		qDebug() << "Downloading message" << job.msgId << "for account"
-		    << job.acntTopIdx.data().toString();
+		qDebug() << "Downloading message" << job.mId.dmId << "for account"
+		    << AccountModel::globAccounts[job.userName].accountName();
 		qDebug() << "-----------------------------------------------";
 
-		if (Q_SUCCESS == downloadMessage(job.acntTopIdx, job.msgId,
-		        true, job.msgDirect, *job.msgDb, errMsg,
+		if (Q_SUCCESS == downloadMessage(job.userName, job.mId,
+		        true, job.msgDirect, *job.dbSet, errMsg,
 		        "DownloadMessage", 0, this)) {
 			/* Only on successful download. */
-			emit refreshAttachmentList(job.acntTopIdx, job.msgId);
+			emit refreshAccountList(job.userName);
+			emit refreshAttachmentList(job.userName, job.mId.dmId);
 		} else {
-			emit clearStatusBarAndShowDialog(job.msgId, errMsg);
+			emit clearStatusBarAndShowDialog(job.mId.dmId, errMsg);
 		}
 
 	} else if (MSG_RECEIVED == job.msgDirect) {
 
+		/* dmStatusFilter
+		 *
+		 * MESSAGESTATE_SENT |
+		 * MESSAGESTATE_STAMPED |
+		 * MESSAGESTATE_DELIVERED |
+		 * MESSAGESTATE_RECEIVED |
+		 * MESSAGESTATE_READ |
+		 * MESSAGESTATE_REMOVED |
+		 * MESSAGESTATE_IN_SAFE |
+		 * MESSAGESTATE_ANY
+		 */
+
 		qDebug() << "-----------------------------------------------";
 		qDebug() << "Downloading received message list for account"
-		    << job.acntTopIdx.data().toString();
+		    << AccountModel::globAccounts[job.userName].accountName();
 		qDebug() << "-----------------------------------------------";
-		res = downloadMessageList(job.acntTopIdx, MSG_RECEIVED,
-		        *job.msgDb, errMsg, "GetListOfReceivedMessages",
-		        0, this, rt, rn);
-		emit refreshAccountList(job.acntTopIdx);
+		QStringList newMsgIdList;
+		res = downloadMessageList(job.userName, MSG_RECEIVED,
+		        *job.dbSet, errMsg, "GetListOfReceivedMessages",
+		        0, this, rt, rn, newMsgIdList, &dmLimit, MESSAGESTATE_ANY);
+		emit refreshAccountList(job.userName);
 		emit changeStatusBarInfo(true, rt, rn , st, sn);
 
 		if (Q_SUCCESS == res) {
@@ -227,11 +243,14 @@ void Worker::doJob(void)
 
 		qDebug() << "-----------------------------------------------";
 		qDebug() << "Downloading sent message list for account"
-		    << job.acntTopIdx.data().toString();
+		    << AccountModel::globAccounts[job.userName].accountName();
 		qDebug() << "-----------------------------------------------";
-		res = downloadMessageList(job.acntTopIdx, MSG_SENT, *job.msgDb,
-		        errMsg, "GetListOfSentMessages", 0, this, st, sn);
-		emit refreshAccountList(job.acntTopIdx);
+
+		QStringList newMsgIdList;
+		res = downloadMessageList(job.userName, MSG_SENT, *job.dbSet,
+		        errMsg, "GetListOfSentMessages", 0, this, st, sn,
+		        newMsgIdList, &dmLimit, MESSAGESTATE_ANY);
+		emit refreshAccountList(job.userName);
 		emit changeStatusBarInfo(true, rt, rn , st, sn);
 
 		if (Q_SUCCESS == res) {
@@ -261,7 +280,7 @@ void Worker::doJob(void)
  * Store envelope into database.
  */
 qdatovka_error Worker::storeEnvelope(enum MessageDirection msgDirect,
-    MessageDb &messageDb, const struct isds_envelope *envel)
+    MessageDbSet &dbSet, const struct isds_envelope *envel)
 /* ========================================================================= */
 {
 	debugFuncCall();
@@ -271,87 +290,68 @@ qdatovka_error Worker::storeEnvelope(enum MessageDirection msgDirect,
 		return Q_GLOBAL_ERROR;
 	}
 
-	qint64 dmId = QString(envel->dmID).toLongLong();
-
-	QString dmAmbiguousRecipient;
-	if (NULL == envel->dmAmbiguousRecipient) {
-		dmAmbiguousRecipient = "0";
-	} else {
-		dmAmbiguousRecipient = QString::number(
-		    *envel->dmAmbiguousRecipient);
+	qint64 dmId = -1;
+	{
+		bool ok = false;
+		dmId = QString(envel->dmID).toLongLong(&ok);
+		if (!ok) {
+			return Q_GLOBAL_ERROR;
+		}
 	}
 
-	QString dmLegalTitleYear;
-	if (NULL != envel->dmLegalTitleYear) {
-		dmLegalTitleYear = QString::number(
-		    *envel->dmLegalTitleYear);
-	}
-
-	QString dmLegalTitleLaw;
-	if (NULL != envel->dmLegalTitleLaw) {
-		dmLegalTitleLaw = QString::number(
-		    *envel->dmLegalTitleLaw);
-	}
-
-	QString dmSenderOrgUnitNum;
-	if ((NULL != envel->dmSenderOrgUnitNum) &&
-	    (0 != *envel->dmSenderOrgUnitNum)) {
-		dmSenderOrgUnitNum = QString::number(
-		    *envel->dmSenderOrgUnitNum);
-	}
-
-	QString dmRecipientOrgUnitNum;
-	if ((NULL != envel->dmRecipientOrgUnitNum) &&
-	    (0 != *envel->dmRecipientOrgUnitNum)) {
-		dmRecipientOrgUnitNum = QString::number(
-		    *envel->dmRecipientOrgUnitNum);
-	}
-
-	QString dmDeliveryTime;
-	if (NULL != envel->dmDeliveryTime) {
-		dmDeliveryTime = timevalToDbFormat(envel->dmDeliveryTime);
-	}
-	QString dmAcceptanceTime;
-	if (NULL != envel->dmAcceptanceTime) {
-		dmAcceptanceTime = timevalToDbFormat(envel->dmAcceptanceTime);
-	}
+	QDateTime deliveryTime = timevalToDateTime(envel->dmDeliveryTime);
+	/* Allow invalid delivery time. */
+	MessageDb *messageDb = dbSet.accessMessageDb(deliveryTime, true);
+	Q_ASSERT(0 != messageDb);
 
 	/* insert message envelope in db */
-	if (messageDb.msgsInsertMessageEnvelope(dmId,
+	if (messageDb->msgsInsertMessageEnvelope(dmId,
 	    /* TODO - set correctly next two values */
 	    "tRecord",
 	    envel->dbIDSender,
 	    envel->dmSender,
 	    envel->dmSenderAddress,
-	    (int) *envel->dmSenderType,
+	    envel->dmSenderType ?
+	        (int) *envel->dmSenderType : 0,
 	    envel->dmRecipient,
 	    envel->dmRecipientAddress,
-	    dmAmbiguousRecipient,
+	    envel->dmAmbiguousRecipient ?
+	        QString::number(*envel->dmAmbiguousRecipient) : QString(),
 	    envel->dmSenderOrgUnit,
-	    dmSenderOrgUnitNum,
+	    (envel->dmSenderOrgUnitNum && *envel->dmSenderOrgUnitNum) ?
+	        QString::number(*envel->dmSenderOrgUnitNum) : QString(),
 	    envel->dbIDRecipient,
 	    envel->dmRecipientOrgUnit,
-	    dmRecipientOrgUnitNum,
+	    (envel->dmRecipientOrgUnitNum && *envel->dmRecipientOrgUnitNum) ?
+	        QString::number(*envel->dmRecipientOrgUnitNum) : QString(),
 	    envel->dmToHands,
 	    envel->dmAnnotation,
 	    envel->dmRecipientRefNumber,
 	    envel->dmSenderRefNumber,
 	    envel->dmRecipientIdent,
 	    envel->dmSenderIdent,
-	    dmLegalTitleLaw,
-	    dmLegalTitleYear,
+	    envel->dmLegalTitleLaw ?
+	        QString::number(*envel->dmLegalTitleLaw) : QString(),
+	    envel->dmLegalTitleYear ?
+	        QString::number(*envel->dmLegalTitleYear) : QString(),
 	    envel->dmLegalTitleSect,
 	    envel->dmLegalTitlePar,
 	    envel->dmLegalTitlePoint,
-	    envel->dmPersonalDelivery,
-	    envel->dmAllowSubstDelivery,
-	    (NULL != envel->timestamp) ?
+	    envel->dmPersonalDelivery ?
+	        *envel->dmPersonalDelivery : false,
+	    envel->dmAllowSubstDelivery ?
+	        *envel->dmAllowSubstDelivery : false,
+	    envel->timestamp ?
 	        QByteArray((char *) envel->timestamp,
 	            envel->timestamp_length).toBase64() : QByteArray(),
-	    dmDeliveryTime,
-	    dmAcceptanceTime,
-	    convertHexToDecIndex(*envel->dmMessageStatus),
-	    (int) *envel->dmAttachmentSize,
+	    envel->dmDeliveryTime ?
+	        timevalToDbFormat(envel->dmDeliveryTime) : QString(),
+	    envel->dmAcceptanceTime ?
+	        timevalToDbFormat(envel->dmAcceptanceTime) : QString(),
+	    envel->dmMessageStatus ?
+	        convertHexToDecIndex(*envel->dmMessageStatus) : 0,
+	    envel->dmAttachmentSize ?
+	        (int) *envel->dmAttachmentSize : 0,
 	    envel->dmType,
 	    msgDirect)) {
 		qDebug() << "Message envelope" << dmId <<
@@ -369,10 +369,11 @@ qdatovka_error Worker::storeEnvelope(enum MessageDirection msgDirect,
 /*
  * Download sent/received message list from ISDS for current account index
  */
-qdatovka_error Worker::downloadMessageList(const QModelIndex &acntTopIdx,
-    enum MessageDirection msgDirect, MessageDb &messageDb, QString &errMsg,
+qdatovka_error Worker::downloadMessageList(const QString &userName,
+    enum MessageDirection msgDirect, MessageDbSet &dbSet, QString &errMsg,
     const QString &progressLabel, QProgressBar *pBar, Worker *worker,
-    int &total, int &news)
+    int &total, int &news, QStringList &newMsgIdList, ulong *dmLimit,
+    int dmStatusFilter)
 /* ========================================================================= */
 {
 #define USE_TRANSACTIONS 1
@@ -381,7 +382,7 @@ qdatovka_error Worker::downloadMessageList(const QModelIndex &acntTopIdx,
 	int newcnt = 0;
 	int allcnt = 0;
 
-	if (!acntTopIdx.isValid()) {
+	if (userName.isEmpty()) {
 		Q_ASSERT(0);
 		return Q_GLOBAL_ERROR;
 	}
@@ -389,40 +390,36 @@ qdatovka_error Worker::downloadMessageList(const QModelIndex &acntTopIdx,
 	if (0 != pBar) { pBar->setValue(0); }
 	if (0 != worker) { emit worker->valueChanged(progressLabel, 0); }
 
-	const AccountModel::SettingsMap accountInfo =
-	    acntTopIdx.data(ROLE_ACNT_CONF_SETTINGS).toMap();
-
 	isds_error status = IE_ERROR;
 
 	if (0 != pBar) { pBar->setValue(10); }
 	if (0 != worker) { emit worker->valueChanged(progressLabel, 10); }
 
+	struct isds_ctx *session = isdsSessions.session(userName);
+	if (NULL == session) {
+		Q_ASSERT(0);
+		return Q_GLOBAL_ERROR;
+	}
 	struct isds_list *messageList = NULL;
 
 	/* Download sent/received message list from ISDS for current account */
 	if (MSG_SENT == msgDirect) {
-		status = isds_get_list_of_sent_messages(isdsSessions.
-		    session(accountInfo.userName()),
+		status = isds_get_list_of_sent_messages(session,
 		    NULL, NULL, NULL,
-		    //MESSAGESTATE_SENT |  MESSAGESTATE_STAMPED |
-		    //MESSAGESTATE_INFECTED | MESSAGESTATE_DELIVERED,
-		    MESSAGESTATE_ANY,
-		    0, NULL, &messageList);
+		    dmStatusFilter,
+		    0, dmLimit, &messageList);
 	} else if (MSG_RECEIVED == msgDirect) {
-		status = isds_get_list_of_received_messages(isdsSessions.
-		    session(accountInfo.userName()),
+		status = isds_get_list_of_received_messages(session,
 		    NULL, NULL, NULL,
-		    //MESSAGESTATE_DELIVERED | MESSAGESTATE_SUBSTITUTED,
-		    MESSAGESTATE_ANY,
-		    0, NULL, &messageList);
+		    dmStatusFilter,
+		    0, dmLimit, &messageList);
 	}
 
 	if (0 != pBar) { pBar->setValue(20); }
 	if (0 != worker) { emit worker->valueChanged(progressLabel, 20); }
 
 	if (status != IE_SUCCESS) {
-		errMsg = isdsLongMessage(isdsSessions.session(
-		    accountInfo.userName()));
+		errMsg = isdsLongMessage(session);
 		qDebug() << status << isds_strerror(status) << errMsg;
 		isds_list_free(&messageList);
 		return Q_ISDS_ERROR;
@@ -447,8 +444,20 @@ qdatovka_error Worker::downloadMessageList(const QModelIndex &acntTopIdx,
 		delta = 80.0 / allcnt;
 	}
 
+	/* Obtain invalid message database if has a separate file. */
+	MessageDb *invalidDb = NULL;
+	{
+		QString invSecKey(dbSet.secondaryKey(QDateTime()));
+		QString valSecKey(dbSet.secondaryKey(
+		    QDateTime::currentDateTime()));
+		if (invSecKey != valSecKey) {
+			/* Invalid database file may not exist. */
+			invalidDb = dbSet.accessMessageDb(QDateTime(), false);
+		}
+	}
+
 #ifdef USE_TRANSACTIONS
-	messageDb.beginTransaction();
+	QSet<MessageDb *> usedDbs;
 #endif /* USE_TRANSACTIONS */
 	while (0 != box) {
 
@@ -465,27 +474,55 @@ qdatovka_error Worker::downloadMessageList(const QModelIndex &acntTopIdx,
 			return Q_ISDS_ERROR;
 		}
 
+		QString dmID = QString(item->envelope->dmID);
 		qint64 dmId = QString(item->envelope->dmID).toLongLong();
+		/*
+		 * Time may be invalid (e.g. messages which failed during
+		 * virus scan).
+		 */
+		QDateTime deliveryTime =
+		    timevalToDateTime(item->envelope->dmDeliveryTime);
+		/* Delivery time may be invalid. */
+		if ((0 != invalidDb) && deliveryTime.isValid()) {
+			/* Try deleting possible invalid entry. */
+			invalidDb->msgsDeleteMessageData(dmId);
+		}
+		MessageDb *messageDb = dbSet.accessMessageDb(deliveryTime, true);
+		Q_ASSERT(0 != messageDb);
 
-		int dmDbMsgStatus = messageDb.msgsStatusIfExists(dmId);
+#ifdef USE_TRANSACTIONS
+		if (!usedDbs.contains(messageDb)) {
+			usedDbs.insert(messageDb);
+			messageDb->beginTransaction();
+		}
+#endif /* USE_TRANSACTIONS */
+
+		const int dmDbMsgStatus = messageDb->msgsStatusIfExists(dmId);
 
 		/* message is not in db (-1) */
 		if (-1 == dmDbMsgStatus) {
-			storeEnvelope(msgDirect, messageDb, item->envelope);
+			storeEnvelope(msgDirect, dbSet, item->envelope);
 
 			if (globPref.auto_download_whole_messages) {
 				QString errMsg;
-				downloadMessage(acntTopIdx, dmId,
-				    true, msgDirect, messageDb, errMsg, "", 0, 0);
+				downloadMessage(userName,
+				    MessageDb::MsgId(dmId, deliveryTime),
+				    true, msgDirect, dbSet, errMsg, "", 0, 0);
 			}
+			newMsgIdList.append(dmID);
 			newcnt++;
 
 		/* Message is in db (dmDbMsgStatus <> -1). */
 		} else {
-			if (MSG_SENT == msgDirect) {
-				int dmNewMsgStatus = convertHexToDecIndex(
-				     *item->envelope->dmMessageStatus);
+			/* Update envelope if message status has changed. */
+			const int dmNewMsgStatus = convertHexToDecIndex(
+			     *item->envelope->dmMessageStatus);
 
+			if (dmNewMsgStatus != dmDbMsgStatus) {
+				updateEnvelope(msgDirect, *messageDb, item->envelope);
+			}
+
+			if (MSG_SENT == msgDirect) {
 				/*
 				 * Sent messages content will be downloaded
 				 * only if those message state is 1 or 2.
@@ -493,23 +530,25 @@ qdatovka_error Worker::downloadMessageList(const QModelIndex &acntTopIdx,
 				if (globPref.auto_download_whole_messages &&
 				    (dmDbMsgStatus <= 2)) {
 					QString errMsg;
-					downloadMessage(acntTopIdx, dmId,
-					    true, msgDirect, messageDb, errMsg,
+					downloadMessage(userName,
+					    MessageDb::MsgId(dmId, deliveryTime),
+					    true, msgDirect, dbSet, errMsg,
 					    "", 0, 0);
 				}
 
 				if (dmDbMsgStatus != dmNewMsgStatus) {
-					getMessageState(msgDirect, acntTopIdx,
-					    dmId, true, messageDb);
+					getMessageState(msgDirect, userName,
+					    dmId, true, dbSet);
 				}
 			}
 
 			/* Message is in db, but the content is missing. */
 			if (globPref.auto_download_whole_messages &&
-			    !messageDb.msgsStoredWhole(dmId)) {
+			    !messageDb->msgsStoredWhole(dmId)) {
 				QString errMsg;
-				downloadMessage(acntTopIdx, dmId,
-				    true, msgDirect, messageDb, errMsg, "", 0, 0);
+				downloadMessage(userName,
+				    MessageDb::MsgId(dmId, deliveryTime),
+				    true, msgDirect, dbSet, errMsg, "", 0, 0);
 			}
 		}
 
@@ -517,7 +556,10 @@ qdatovka_error Worker::downloadMessageList(const QModelIndex &acntTopIdx,
 
 	}
 #ifdef USE_TRANSACTIONS
-	messageDb.commitTransaction();
+	/* Commit on all opened databases. */
+	foreach (MessageDb *db, usedDbs) {
+		db->commitTransaction();
+	}
 #endif /* USE_TRANSACTIONS */
 
 	isds_list_free(&messageList);
@@ -526,11 +568,10 @@ qdatovka_error Worker::downloadMessageList(const QModelIndex &acntTopIdx,
 	if (0 != worker) { emit worker->valueChanged(progressLabel, 100); }
 
 	if (MSG_RECEIVED == msgDirect) {
-		qDebug() << "#Received total:" << allcnt;
-		qDebug() << "#Received new:" << newcnt;
+		qDebug() << "#Received total:" << allcnt
+		    << " #Received new:" << newcnt;
 	} else {
-		qDebug() << "#Sent total:" << allcnt;
-		qDebug() << "#Sent new:" << newcnt;
+		qDebug() << "#Sent total:" << allcnt << " #Sent new:" << newcnt;
 	}
 
 	total = allcnt;
@@ -546,7 +587,7 @@ qdatovka_error Worker::downloadMessageList(const QModelIndex &acntTopIdx,
  * Store sent message delivery information into database.
  */
 qdatovka_error Worker::updateMessageState(enum MessageDirection msgDirect,
-    MessageDb &messageDb, const struct isds_envelope *envel)
+    MessageDbSet &dbSet, const struct isds_envelope *envel)
 /* ========================================================================= */
 {
 	if (NULL == envel) {
@@ -554,7 +595,18 @@ qdatovka_error Worker::updateMessageState(enum MessageDirection msgDirect,
 		return Q_GLOBAL_ERROR;
 	}
 
-	qint64 dmID = QString(envel->dmID).toLongLong();
+	qint64 dmID = -1;
+	{
+		bool ok = false;
+		dmID = QString(envel->dmID).toLongLong(&ok);
+		if (!ok) {
+			return Q_GLOBAL_ERROR;
+		}
+	}
+	QDateTime deliveryTime = timevalToDateTime(envel->dmDeliveryTime);
+	Q_ASSERT(deliveryTime.isValid());
+	MessageDb *messageDb = dbSet.accessMessageDb(deliveryTime, true);
+	Q_ASSERT(0 != messageDb);
 
 	QString dmDeliveryTime;
 	if (NULL != envel->dmDeliveryTime) {
@@ -565,7 +617,7 @@ qdatovka_error Worker::updateMessageState(enum MessageDirection msgDirect,
 		dmAcceptanceTime = timevalToDbFormat(envel->dmAcceptanceTime);
 	}
 
-	if (1 == messageDb.messageState(dmID)) {
+	if (1 == messageDb->messageState(dmID)) {
 		/*
 		 * Update message envelope when the previous state
 		 * is 1. This is because the envelope was generated by this
@@ -573,10 +625,11 @@ qdatovka_error Worker::updateMessageState(enum MessageDirection msgDirect,
 		 * we get proper data from ISDS rather than storing potentially
 		 * guessed values.
 		 */
-		updateEnvelope(msgDirect, messageDb, envel);
-	} else if (messageDb.msgsUpdateMessageState(dmID,
+		updateEnvelope(msgDirect, *messageDb, envel);
+	} else if (messageDb->msgsUpdateMessageState(dmID,
 	    dmDeliveryTime, dmAcceptanceTime,
-	    convertHexToDecIndex(*envel->dmMessageStatus))) {
+	    envel->dmMessageStatus ?
+	        convertHexToDecIndex(*envel->dmMessageStatus) : 0)) {
 		/* Updated message envelope delivery info in db. */
 		qDebug() << "Message envelope delivery info was updated...";
 	} else {
@@ -588,7 +641,7 @@ qdatovka_error Worker::updateMessageState(enum MessageDirection msgDirect,
 
 	while (0 != event) {
 		isds_event *item = (isds_event *) event->data;
-		messageDb.msgsInsertUpdateMessageEvent(dmID,
+		messageDb->msgsInsertUpdateMessageEvent(dmID,
 		    timevalToDbFormat(item->time),
 		    convertEventTypeToString(*item->type),
 		    item->description);
@@ -604,29 +657,29 @@ qdatovka_error Worker::updateMessageState(enum MessageDirection msgDirect,
 * Download sent message delivery info and get list of events message
 */
 bool Worker::getMessageState(enum MessageDirection msgDirect,
-    const QModelIndex &acntTopIdx, qint64 dmId, bool signedMsg,
-    MessageDb &messageDb)
+    const QString &userName, qint64 dmId, bool signedMsg,
+    MessageDbSet &dbSet)
 /* ========================================================================= */
 {
 	debugFuncCall();
 
-	const AccountModel::SettingsMap accountInfo =
-	    acntTopIdx.data(ROLE_ACNT_CONF_SETTINGS).toMap();
-
 	isds_error status = IE_ERROR;
 
+	struct isds_ctx *session = isdsSessions.session(userName);
+	if (NULL == session) {
+		Q_ASSERT(0);
+		return false;
+	}
 	struct isds_message *message = NULL;
 
 	if (signedMsg) {
-		status = isds_get_signed_delivery_info(
-		    isdsSessions.session(accountInfo.userName()),
+		status = isds_get_signed_delivery_info(session,
 		    QString::number(dmId).toUtf8().constData(), &message);
 	} else {
 		Q_ASSERT(0); /* Only signed messages can be downloaded. */
 		goto fail;
 		/*
-		status = isds_get_delivery_info(
-		    isdsSessions.session(accountInfo.userName()),
+		status = isds_get_delivery_info(session,
 		    QString::number(dmId).toUtf8().constData(), &message);
 		*/
 	}
@@ -639,7 +692,7 @@ bool Worker::getMessageState(enum MessageDirection msgDirect,
 	Q_ASSERT(NULL != message);
 
 	if (Q_SUCCESS !=
-	    updateMessageState(msgDirect, messageDb, message->envelope)) {
+	    updateMessageState(msgDirect, dbSet, message->envelope)) {
 		goto fail;
 	}
 
@@ -662,7 +715,7 @@ fail:
  */
 qdatovka_error Worker::storeMessage(bool signedMsg,
     enum MessageDirection msgDirect,
-    MessageDb &messageDb, const struct isds_message *msg,
+    MessageDbSet &dbSet, const struct isds_message *msg,
     const QString &progressLabel, QProgressBar *pBar, Worker *worker)
 /* ========================================================================= */
 {
@@ -685,11 +738,30 @@ qdatovka_error Worker::storeMessage(bool signedMsg,
 		return Q_GLOBAL_ERROR;
 	}
 
-	qint64 dmID = QString(envel->dmID).toLongLong();
+	qint64 dmID = -1;
+	{
+		bool ok = false;
+		dmID = QString(envel->dmID).toLongLong(&ok);
+		if (!ok) {
+			return Q_GLOBAL_ERROR;
+		}
+	}
+	QDateTime deliveryTime = timevalToDateTime(envel->dmDeliveryTime);
+	Q_ASSERT(deliveryTime.isValid());
+	MessageDb *messageDb = dbSet.accessMessageDb(deliveryTime, true);
+	Q_ASSERT(0 != messageDb);
+
+	/*
+	 * If there is no raw message then all the attachments have been
+	 * stored when the message has been set.
+	 */
+	if (!messageDb->msgsStoredWhole(dmID)) {
+		messageDb->flsDeleteMessageFiles(dmID);
+	}
 
 	/* Get signed raw data from message and store to db. */
 	if (signedMsg) {
-		(messageDb.msgsInsertUpdateMessageRaw(dmID,
+		(messageDb->msgsInsertUpdateMessageRaw(dmID,
 		    QByteArray((char*) msg->raw, msg->raw_length), 0))
 		? qDebug() << "Message raw data were updated..."
 		: qDebug() << "ERROR: Message raw data update!";
@@ -698,7 +770,7 @@ qdatovka_error Worker::storeMessage(bool signedMsg,
 	if (0 != pBar) { pBar->setValue(30); }
 	if (0 != worker) { emit worker->valueChanged(progressLabel, 30); }
 
-	if (updateEnvelope(msgDirect, messageDb, envel)) {
+	if (updateEnvelope(msgDirect, *messageDb, envel)) {
 		qDebug() << "Message envelope was updated...";
 	} else {
 		qDebug() << "ERROR: Message envelope update!";
@@ -713,12 +785,10 @@ qdatovka_error Worker::storeMessage(bool signedMsg,
 		    msg->raw_length, 1, globPref.check_crl ? 1 : 0);
 		qDebug() << "Verification ret" << ret;
 		if (1 == ret) {
-			messageDb.msgsSetVerified(dmID,
-			    true);
+			messageDb->msgsSetVerified(dmID, true);
 			/* TODO -- handle return error. */
 		} else if (0 == ret){
-			messageDb.msgsSetVerified(dmID,
-			    false);
+			messageDb->msgsSetVerified(dmID, false);
 			/* TODO -- handle return error. */
 		} else {
 			/* TODO -- handle this error. */
@@ -734,7 +804,7 @@ qdatovka_error Worker::storeMessage(bool signedMsg,
 
 		QByteArray hashValueBase64 = QByteArray((char *) hash->value,
 		    hash->length).toBase64();
-		if (messageDb.msgsInsertUpdateMessageHash(dmID,
+		if (messageDb->msgsInsertUpdateMessageHash(dmID,
 		        hashValueBase64, convertHashAlg(hash->algorithm))) {
 			qDebug() << "Message hash was stored into db...";
 		} else {
@@ -746,20 +816,31 @@ qdatovka_error Worker::storeMessage(bool signedMsg,
 	if (0 != worker) { emit worker->valueChanged(progressLabel, 70); }
 
 	/* Insert/update all attachment files */
-	const struct isds_list *file = msg->documents;
-	while (0 != file) {
+	storeAttachments(*messageDb, dmID, msg->documents);
+
+	return Q_SUCCESS;
+}
+
+
+/* ========================================================================= */
+qdatovka_error Worker::storeAttachments(MessageDb &db, qint64 dmID,
+    const struct isds_list *documents)
+/* ========================================================================= */
+{
+	const struct isds_list *file = documents;
+
+	while (NULL != file) {
 		const isds_document *item = (isds_document *) file->data;
 
-		QByteArray dmEncodedContentBase64 =
-		    QByteArray((char *)item->data,
-		        item->data_length).toBase64();
+		QByteArray dmEncodedContentBase64 = QByteArray(
+		    (char *)item->data, item->data_length).toBase64();
 
 		/* Insert/update file to db */
-		if (messageDb.msgsInsertUpdateMessageFile(dmID,
-		   item->dmFileDescr, item->dmUpFileGuid, item->dmFileGuid,
-		   item->dmMimeType, item->dmFormat,
-		   convertAttachmentType(item->dmFileMetaType),
-		   dmEncodedContentBase64)) {
+		if (db.msgsInsertUpdateMessageFile(dmID, item->dmFileDescr,
+		        item->dmUpFileGuid, item->dmFileGuid, item->dmMimeType,
+		        item->dmFormat,
+		        convertAttachmentType(item->dmFileMetaType),
+		        dmEncodedContentBase64)) {
 			qDebug() << "Message file" << item->dmFileDescr
 			    << "was stored into db...";
 		} else {
@@ -777,24 +858,26 @@ qdatovka_error Worker::storeMessage(bool signedMsg,
 /*
  * Download attachments, envelope and raw for message.
  */
-qdatovka_error Worker::downloadMessage(const QModelIndex &acntTopIdx,
-    qint64 dmId, bool signedMsg, enum MessageDirection msgDirect,
-    MessageDb &messageDb, QString &errMsg, const QString &progressLabel, QProgressBar *pBar,
+qdatovka_error Worker::downloadMessage(const QString &userName,
+    MessageDb::MsgId mId, bool signedMsg, enum MessageDirection msgDirect,
+    MessageDbSet &dbSet, QString &errMsg, const QString &progressLabel, QProgressBar *pBar,
     Worker *worker)
 /* ========================================================================= */
 {
 	debugFuncCall();
 
-	qDebug() << "Downloading complete message" << dmId;
+	qDebug() << "Downloading complete message" << mId.dmId;
 
 	if (0 != pBar) { pBar->setValue(0); }
 	if (0 != worker) { emit worker->valueChanged(progressLabel, 0); }
 
-	const AccountModel::SettingsMap accountInfo =
-	    acntTopIdx.data(ROLE_ACNT_CONF_SETTINGS).toMap();
-
 	isds_error status;
 
+	struct isds_ctx *session = isdsSessions.session(userName);
+	if (NULL == session) {
+		Q_ASSERT(0);
+		return Q_GLOBAL_ERROR;
+	}
 	// message structures - all members
 	struct isds_message *message = NULL;
 
@@ -806,13 +889,13 @@ qdatovka_error Worker::downloadMessage(const QModelIndex &acntTopIdx,
 		/* sent or received message? */
 		if (MSG_RECEIVED == msgDirect) {
 			status = isds_get_signed_received_message(
-			    isdsSessions.session(accountInfo.userName()),
-			    QString::number(dmId).toUtf8().constData(),
+			    isdsSessions.session(userName),
+			    QString::number(mId.dmId).toUtf8().constData(),
 			    &message);
 		} else {
 			status = isds_get_signed_sent_message(
-			    isdsSessions.session(accountInfo.userName()),
-			    QString::number(dmId).toUtf8().constData(),
+			    isdsSessions.session(userName),
+			    QString::number(mId.dmId).toUtf8().constData(),
 			    &message);
 		}
 	} else {
@@ -820,8 +903,8 @@ qdatovka_error Worker::downloadMessage(const QModelIndex &acntTopIdx,
 		return Q_GLOBAL_ERROR;
 		/*
 		status = isds_get_received_message(isdsSessions.session(
-		    accountInfo.userName()),
-		    QString::number(dmId).toUtf8().constData(),
+		    userName),
+		    QString::number(mId.dmId).toUtf8().constData(),
 		    &message);
 		*/
 	}
@@ -830,34 +913,64 @@ qdatovka_error Worker::downloadMessage(const QModelIndex &acntTopIdx,
 	if (0 != worker) { emit worker->valueChanged(progressLabel, 20); }
 
 	if (IE_SUCCESS != status) {
-		errMsg = isdsLongMessage(
-		    isdsSessions.session(accountInfo.userName()));
+		errMsg = isdsLongMessage(session);
 		qDebug() << status << isds_strerror(status) << errMsg;
 		isds_message_free(&message);
 		return Q_ISDS_ERROR;
 	}
 
-	/* Download and store the message. */
-	storeMessage(signedMsg, msgDirect, messageDb, message,
+	{
+		QString secKeyBeforeDownload(dbSet.secondaryKey(mId.deliveryTime));
+		QDateTime newDeliveryTime(timevalToDateTime(
+		    message->envelope->dmDeliveryTime));
+		QString secKeyAfterDonwload(dbSet.secondaryKey(newDeliveryTime));
+		/*
+		 * Secondary keys may be the sam for valid and invalid
+		 * delivery time when storing into single file.
+		 */
+		if (secKeyBeforeDownload != secKeyAfterDonwload) {
+			/*
+			 * The message was likely moved from invalid somewhere
+			 * else.
+			 */
+			/* Delete the message from invalid. */
+			MessageDb *db = dbSet.accessMessageDb(mId.deliveryTime, false);
+			if (0 != db) {
+				db->msgsDeleteMessageData(mId.dmId);
+			}
+
+			/* Store envelope in new location. */
+			storeEnvelope(msgDirect, dbSet, message->envelope);
+		}
+		/* Update message delivery time. */
+		mId.deliveryTime = newDeliveryTime;
+	}
+
+	/* Store the message. */
+	storeMessage(signedMsg, msgDirect, dbSet, message,
 	    progressLabel, pBar, worker);
 
 	if (0 != pBar) { pBar->setValue(90); }
 	if (0 != worker) { emit worker->valueChanged(progressLabel, 90); }
 
-	Q_ASSERT(dmId == QString(message->envelope->dmID).toLongLong());
+	Q_ASSERT(mId.dmId == QString(message->envelope->dmID).toLongLong());
 
 	/* Download and save delivery info and message events */
-	if (getDeliveryInfo(acntTopIdx, dmId, signedMsg, messageDb)) {
+	if (Q_SUCCESS == getDeliveryInfo(userName, mId.dmId, signedMsg, dbSet)) {
 		qDebug() << "Delivery info of message was processed...";
 	} else {
 		qDebug() << "ERROR: Delivery info of message not found!";
 	}
 
-	getMessageAuthor(acntTopIdx, dmId, messageDb);
+	Q_ASSERT(mId.deliveryTime.isValid());
+	MessageDb *messageDb = dbSet.accessMessageDb(mId.deliveryTime, true);
+	Q_ASSERT(0 != messageDb);
+
+	getMessageAuthor(userName, mId.dmId, *messageDb);
 
 	if (MSG_RECEIVED == msgDirect) {
 		/*  Mark this message as downloaded in ISDS */
-		if (markMessageAsDownloaded(acntTopIdx, dmId)) {
+		if (markMessageAsDownloaded(userName, mId.dmId)) {
 			qDebug() << "Message was marked as downloaded...";
 		} else {
 			qDebug() << "ERROR: Message was not marked as downloaded!";
@@ -881,7 +994,7 @@ qdatovka_error Worker::downloadMessage(const QModelIndex &acntTopIdx,
  * Store received message delivery information into database.
  */
 qdatovka_error Worker::storeDeliveryInfo(bool signedMsg,
-    MessageDb &messageDb, const struct isds_message *msg)
+    MessageDbSet &dbSet, const struct isds_message *msg)
 /* ========================================================================= */
 {
 	if (NULL == msg) {
@@ -897,10 +1010,14 @@ qdatovka_error Worker::storeDeliveryInfo(bool signedMsg,
 	}
 
 	qint64 dmID = QString(envel->dmID).toLongLong();
+	QDateTime deliveryTime = timevalToDateTime(envel->dmDeliveryTime);
+	Q_ASSERT(deliveryTime.isValid());
+	MessageDb *messageDb = dbSet.accessMessageDb(deliveryTime, true);
+	Q_ASSERT(0 != messageDb);
 
 	/* get signed raw data from message */
 	if (signedMsg) {
-		if (messageDb.msgsInsertUpdateDeliveryInfoRaw(dmID,
+		if (messageDb->msgsInsertUpdateDeliveryInfoRaw(dmID,
 		    QByteArray((char*)msg->raw, msg->raw_length))) {
 			qDebug() << "Message raw delivery info was updated...";
 		} else {
@@ -913,7 +1030,7 @@ qdatovka_error Worker::storeDeliveryInfo(bool signedMsg,
 
 	while (0 != event) {
 		isds_event *item = (isds_event *) event->data;
-		messageDb.msgsInsertUpdateMessageEvent(dmID,
+		messageDb->msgsInsertUpdateMessageEvent(dmID,
 		    timevalToDbFormat(item->time),
 		    convertEventTypeToString(*item->type),
 		    item->description);
@@ -928,30 +1045,30 @@ qdatovka_error Worker::storeDeliveryInfo(bool signedMsg,
 /*
 * Download message delivery info, raw and get list of events message
 */
-bool Worker::getDeliveryInfo(const QModelIndex &acntTopIdx,
-    qint64 dmId, bool signedMsg, MessageDb &messageDb)
+bool Worker::getDeliveryInfo(const QString &userName,
+    qint64 dmId, bool signedMsg, MessageDbSet &dbSet)
 /* ========================================================================= */
 {
 	debugFuncCall();
 
-	const AccountModel::SettingsMap accountInfo =
-	    acntTopIdx.data(ROLE_ACNT_CONF_SETTINGS).toMap();
-
 	isds_error status;
 
+	struct isds_ctx *session = isdsSessions.session(userName);
+	if (NULL == session) {
+		Q_ASSERT(0);
+		return Q_GLOBAL_ERROR;
+	}
 	struct isds_message *message = NULL;
 
 	if (signedMsg) {
-		status = isds_get_signed_delivery_info(isdsSessions.session(
-		    accountInfo.userName()),
+		status = isds_get_signed_delivery_info(session,
 		    QString::number(dmId).toUtf8().constData(),
 		    &message);
 	} else {
 		Q_ASSERT(0); /* Only signed messages can be downloaded. */
 		goto fail;
 		/*
-		status = isds_get_delivery_info(isdsSessions.session(
-		    accountInfo.userName()),
+		status = isds_get_delivery_info(session,
 		    QString::number(dmId).toUtf8().constData(),
 		    &message);
 		*/
@@ -964,20 +1081,20 @@ bool Worker::getDeliveryInfo(const QModelIndex &acntTopIdx,
 
 	Q_ASSERT(NULL != message);
 
-	if (Q_SUCCESS != storeDeliveryInfo(signedMsg, messageDb, message)) {
+	if (Q_SUCCESS != storeDeliveryInfo(signedMsg, dbSet, message)) {
 		goto fail;
 	}
 
 	isds_message_free(&message);
 
-	return true;
+	return Q_SUCCESS;
 
 fail:
 	if (NULL != message) {
 		isds_message_free(&message);
 	}
 
-	return false;
+	return Q_ISDS_ERROR;
 }
 
 
@@ -985,21 +1102,23 @@ fail:
 /*
 * Get additional info about author (sender)
 */
-bool Worker::getMessageAuthor(const QModelIndex &acntTopIdx,
+bool Worker::getMessageAuthor(const QString &userName,
     qint64 dmId, MessageDb &messageDb)
 /* ========================================================================= */
 {
-	const AccountModel::SettingsMap accountInfo =
-	    acntTopIdx.data(ROLE_ACNT_CONF_SETTINGS).toMap();
-
 	isds_error status;
 
 	isds_sender_type *sender_type = NULL;
 	char * raw_sender_type = NULL;
 	char * sender_name = NULL;
 
-	status = isds_get_message_sender(isdsSessions.session(
-	    accountInfo.userName()),
+	struct isds_ctx *session = isdsSessions.session(userName);
+	if (NULL == session) {
+		Q_ASSERT(0);
+		return false;
+	}
+
+	status = isds_get_message_sender(session,
 	    QString::number(dmId).toUtf8().constData(),
 	    &sender_type, &raw_sender_type, &sender_name);
 
@@ -1023,19 +1142,20 @@ bool Worker::getMessageAuthor(const QModelIndex &acntTopIdx,
 /*
 * Set message as downloaded from ISDS.
 */
-bool Worker::markMessageAsDownloaded(const QModelIndex &acntTopIdx,
-    qint64 dmId)
+bool Worker::markMessageAsDownloaded(const QString &userName, qint64 dmId)
 /* ========================================================================= */
 {
 	debugFuncCall();
 
-	const AccountModel::SettingsMap accountInfo =
-	    acntTopIdx.data(ROLE_ACNT_CONF_SETTINGS).toMap();
+	struct isds_ctx *session = isdsSessions.session(userName);
+	if (NULL == session) {
+		Q_ASSERT(0);
+		return false;
+	}
 
 	isds_error status;
 
-	status = isds_mark_message_read(isdsSessions.session(
-	    accountInfo.userName()),
+	status = isds_mark_message_read(session,
 	    QString::number(dmId).toUtf8().constData());
 
 	if (IE_SUCCESS != status) {
@@ -1060,78 +1180,62 @@ qdatovka_error Worker::updateEnvelope(enum MessageDirection msgDirect,
 		return Q_GLOBAL_ERROR;
 	}
 
-	qint64 dmID = QString(envel->dmID).toLongLong();
-
-	QString dmAmbiguousRecipient;
-	if (NULL != envel->dmAmbiguousRecipient) {
-		dmAmbiguousRecipient = QString::number(
-		    *envel->dmAmbiguousRecipient);
-	}
-	QString dmLegalTitleYear;
-	if (NULL != envel->dmLegalTitleYear) {
-		dmLegalTitleYear = QString::number(*envel->dmLegalTitleYear);
-	}
-	QString dmLegalTitleLaw;
-	if (NULL != envel->dmLegalTitleLaw) {
-		dmLegalTitleLaw = QString::number(*envel->dmLegalTitleLaw);
-	}
-	QString dmSenderOrgUnitNum;
-	if ((NULL != envel->dmSenderOrgUnitNum) &&
-	    (0 != *envel->dmSenderOrgUnitNum)) {
-		dmSenderOrgUnitNum = QString::number(
-		    *envel->dmSenderOrgUnitNum);
-	}
-	QString dmRecipientOrgUnitNum;
-	if ((NULL != envel->dmRecipientOrgUnitNum) &&
-	    (0 != *envel->dmRecipientOrgUnitNum)) {
-		dmRecipientOrgUnitNum = QString::number(
-		    *envel->dmRecipientOrgUnitNum);
-	}
-	QString dmDeliveryTime;
-	if (NULL != envel->dmDeliveryTime) {
-		dmDeliveryTime = timevalToDbFormat(envel->dmDeliveryTime);
-	}
-	QString dmAcceptanceTime;
-	if (NULL != envel->dmAcceptanceTime) {
-		dmAcceptanceTime = timevalToDbFormat(envel->dmAcceptanceTime);
+	qint64 dmId = -1;
+	{
+		bool ok = false;
+		dmId = QString(envel->dmID).toLongLong(&ok);
+		if (!ok) {
+			return Q_GLOBAL_ERROR;
+		}
 	}
 
 	/* Update message envelope in db. */
-	if (messageDb.msgsUpdateMessageEnvelope(dmID,
+	if (messageDb.msgsUpdateMessageEnvelope(dmId,
 	    /* TODO - set correctly next two values */
 	    "tReturnedMessage",
 	    envel->dbIDSender,
 	    envel->dmSender,
 	    envel->dmSenderAddress,
-	    (int) *envel->dmSenderType,
+	    envel->dmSenderType ?
+	        (int) *envel->dmSenderType : 0,
 	    envel->dmRecipient,
 	    envel->dmRecipientAddress,
-	    dmAmbiguousRecipient,
+	    envel->dmAmbiguousRecipient ?
+	        QString::number(*envel->dmAmbiguousRecipient) : QString(),
 	    envel->dmSenderOrgUnit,
-	    dmSenderOrgUnitNum,
+	    (envel->dmSenderOrgUnitNum && *envel->dmSenderOrgUnitNum) ?
+	        QString::number(*envel->dmSenderOrgUnitNum) : QString(),
 	    envel->dbIDRecipient,
 	    envel->dmRecipientOrgUnit,
-	    dmRecipientOrgUnitNum,
+	    (envel->dmRecipientOrgUnitNum && *envel->dmRecipientOrgUnitNum) ?
+	        QString::number(*envel->dmRecipientOrgUnitNum) : QString(),
 	    envel->dmToHands,
 	    envel->dmAnnotation,
 	    envel->dmRecipientRefNumber,
 	    envel->dmSenderRefNumber,
 	    envel->dmRecipientIdent,
 	    envel->dmSenderIdent,
-	    dmLegalTitleLaw,
-	    dmLegalTitleYear,
+	    envel->dmLegalTitleLaw ?
+	        QString::number(*envel->dmLegalTitleLaw) : QString(),
+	    envel->dmLegalTitleYear ?
+	        QString::number(*envel->dmLegalTitleYear) : QString(),
 	    envel->dmLegalTitleSect,
 	    envel->dmLegalTitlePar,
 	    envel->dmLegalTitlePoint,
-	    envel->dmPersonalDelivery,
-	    envel->dmAllowSubstDelivery,
-	    (NULL != envel->timestamp) ?
+	    envel->dmPersonalDelivery ?
+	        *envel->dmPersonalDelivery : false,
+	    envel->dmAllowSubstDelivery ?
+	        *envel->dmAllowSubstDelivery : false,
+	    envel->timestamp ?
 	        QByteArray((char *) envel->timestamp,
 	            envel->timestamp_length).toBase64() : QByteArray(),
-	    dmDeliveryTime,
-	    dmAcceptanceTime,
-	    convertHexToDecIndex(*envel->dmMessageStatus),
-	    (NULL != envel->dmAttachmentSize) ?
+	    envel->dmDeliveryTime ?
+	        timevalToDbFormat(envel->dmDeliveryTime) : QString(),
+	    envel->dmAcceptanceTime ?
+	        timevalToDbFormat(envel->dmAcceptanceTime) : QString(),
+	    envel->dmMessageStatus ?
+	        convertHexToDecIndex(*envel->dmMessageStatus) : 0,
+	    envel->dmAttachmentSize ?
 	        (int) *envel->dmAttachmentSize : 0,
 	    envel->dmType,
 	    msgDirect)) {
